var Categorie =
{
    dataReceivedCallback: null,
    internalCallback: null
};

Categorie.loadAllCategories = function () {

    //recursive add servers
    Main.debug("[Categorie.js] - Getting Servers");

    //clear everything
    Data.categories = [];

    //add the default categories
    this.loadDefaults();
    this.refresh();

    Main.debug("[Categorie.js] - Load Finished");

    if (Categorie.dataReceivedCallback) {
        Main.startup = false;
        Categorie.dataReceivedCallback();
    }

    setTimeout(Categorie.loadServers, 300);

};

Categorie.loadServers = function () {
    var url;
    var i;
    var server;


    if (!MyPlex.loggedIn) {
        //remove any server that needs MyPlex
        for (i = 0; i < prefs.servers.length; i++) {
            server = new Server(prefs.servers[i].machineIdentifier);
            if (server.owned && server.synced) {
                prefs.servers.splice(i, 1);
            }
        }
    }

    //first try the primary adresses
    for (i = 0; i < prefs.servers.length; i++) {
        server = new Server(prefs.servers[i].machineIdentifier);
        Main.debug("[Categorie.js] - server.machineIdentifier: " + server.machineIdentifier);
        if (server.owned) {
            prefs.servers[i].reachable = "no"; //reset indication
            url = server.getUrl(0) + "/";
            url = server.appendToken(url);
            setTimeout(Categorie.checkServer, 0, url, i, Categorie.addServerCategories);
        }
    }

    //wait for the callbacks to complete and check if we need more searching
    //no need to search the rest when we are not logged in
    if (MyPlex.loggedIn) {
        //then try the remote ones
        for (i = 0; i < prefs.servers.length; i++) {
            server = new Server(prefs.servers[i].machineIdentifier);
            //update all servers to get the latest status
            if (!server.reachable) {
                url = server.getRemoteAddress() + "/";
                url = server.appendToken(url);
                setTimeout(Categorie.checkServer, 5000, url, i, Categorie.addServerCategories);
            }
        }
    }
};

Categorie.checkServer = function (url, serverIdx, callback) {

    if (prefs.servers[serverIdx].reachable === "yes") {
        //already checked with success, don't bother
        return false;
    }

    Main.debug("[Categorie.js] - Checking Server: " + prefs.servers[serverIdx].name + ", on " + url);

    loadbar.show();

    if (!url) {
        callback(serverIdx, false);
        return false;
    }

    //this needs to be async without a queue to be fast

    $.ajax({
        url: url,
        type: "GET",
        timeout: 10000,
        beforeSend: function (xhr) {
            xhr.id = serverIdx;
            xhr.url = url;
            xhr.callback = Categorie.refresh();
        },
        success: function (xml, textStatus, xhr) {

            if (xhr) {
                var id = xhr.id;

                if (prefs.servers[serverIdx].reachable === "yes") {
                    //already checked with success, don't bother
                    return false;
                }

                if (xhr.responseXML) {
                    var xmldoc = xhr.responseXML.documentElement;
                } else {
                    return false;
                }

                var uri = (xhr.url.contains('?')) ? xhr.url.leftFrom("?") : xhr.url; //this ip works, update it
                var ip = uri.between("http://", ":");
                var port = uri.rightFromLast(":");

                if (port.endsWith("/")) {
                    port = port.leftFromLast("/");
                }

                prefs.servers[id].url = uri;
                prefs.servers[id].ip = ip;
                prefs.servers[id].port = port;
                prefs.servers[id].reachable = "yes";

                if (xmldoc) {

                    var tree = ObjTree.parseDOM(xhr.responseXML.documentElement); //build the object from the xml

                    if (tree) {
                        if (tree.MediaContainer) {
                            prefs.servers[id].allowMediaDeletion = Number(tree.MediaContainer["-allowMediaDeletion"] || 0);
                            prefs.servers[id].flashInstalled = Number(tree.MediaContainer["-flashInstalled"] || 0);
                            prefs.servers[id].friendlyName = (tree.MediaContainer["-friendlyName"] || tree.MediaContainer["-serverName"] || "").replaceHtml();
                            prefs.servers[id].machineIdentifier = (tree.MediaContainer["-machineIdentifier"] || "");
                            prefs.servers[id].multiuser = Number(tree.MediaContainer["-multiuser"] || 0);
                            prefs.servers[id].platform = (tree.MediaContainer["-platform"] || "");
                            prefs.servers[id].platformVersion = (tree.MediaContainer["-platformVersion"] || "");
                            prefs.servers[id].silverlightInstalled = Number(tree.MediaContainer["-silverlightInstalled"] || 0);
                            prefs.servers[id].transcoderActiveVideoSessions = Number(tree.MediaContainer["-transcoderActiveVideoSessions"] || 0);
                            prefs.servers[id].transcoderAudio = Number(tree.MediaContainer["-transcoderAudio"] || 0);
                            prefs.servers[id].transcoderVideo = Number(tree.MediaContainer["-transcoderVideo"] || 0);
                            prefs.servers[id].transcoderVideoBitrates = (tree.MediaContainer["-transcoderVideoBitrates"] || "");
                            prefs.servers[id].transcoderVideoQualities = (tree.MediaContainer["-transcoderVideoQualities"] || "");
                            prefs.servers[id].transcoderVideoResolutions = (tree.MediaContainer["-transcoderVideoResolutions"] || "");
                            prefs.servers[id].version = (tree.MediaContainer["-version"] || "");
                            prefs.servers[id].webkit = Number(tree.MediaContainer["-webkit"] || 0);

                            if (tree.MediaContainer.Directory) {
                                for (var i = 0; i < tree.MediaContainer.Directory.length; i++) {

                                    if (tree.MediaContainer.Directory[i]["-key"] === "video") {
                                        prefs.servers[id].videoCount = Number(tree.MediaContainer.Directory[i]["-count"] || 0);
                                    }
                                    if (tree.MediaContainer.Directory[i]["-key"] === "music") {
                                        prefs.servers[id].musicCount = Number(tree.MediaContainer.Directory[i]["-count"] || 0);
                                    }
                                    if (tree.MediaContainer.Directory[i]["-key"] === "photos") {
                                        prefs.servers[id].photosCount = Number(tree.MediaContainer.Directory[i]["-count"] || 0);
                                    }
                                    if (tree.MediaContainer.Directory[i]["-key"] === "applications") {
                                        prefs.servers[id].applicationsCount = Number(tree.MediaContainer.Directory[i]["-count"] || 0);
                                    }
                                }
                            }
                        }
                    }
                }

                Main.debug("[Categorie.js] - Found server " + prefs.servers[id].name + ", to be reachable on " + prefs.servers[id].url);

                if ($('#splash').is(':visible') && Main.startup === true) {
                    $("#loadingProgress").putInnerHTML(text.connectServer + "<br>" + prefs.servers[id].name).show();
                }

                callback(xhr.id, true);

            } else {
                callback(xhr.id, false);
            }

        },
        error: function (xhr) {
            if (xhr.id) {
                if (prefs.servers[xhr.id]) {
                    Main.debug("[Categorie.js] - Server check: " + prefs.servers[xhr.id].name + " : " + url + " -> not reachable");
                    prefs.servers[xhr.id].reachable = "no";
                }
                callback(xhr.id, false);
            }
        }
    });
};

Categorie.addServerCategories = function (id, found) {
    if (found) {
        var server = new Server(prefs.servers[id].machineIdentifier);
        //only add owned servers
        if (server.owned) {
            var url = server.url.join("/library/sections/");
            url = server.appendToken(url);
            //Main.debug("[Categorie.js] - Load library: " + url);
            AjaxManager.parseHTTP(url, Categorie.createCategorieList, null, 0, prefs.servers[id].machineIdentifier);
        }
    }
};

Categorie.createCategorieList = function (tree, xhrObj, id) {

    Main.debug("Catergorie.js - createCategorieList for machID: " + id);

    var i = Data.categories.length;
    var server = new Server(id);
    var cat = {};

    Main.debug("Catergorie.js - server videocount: " + server.videoCount);

    if (xhrObj === null) //timeout occured
    {
        Categorie.handleError(null);
    }
    else if (xhrObj.status !== 200) {
        Categorie.handleError(xhrObj);
    }
    else {
        xhrObj = null;
    }

    //add local library and channels
    if (tree) {

        if (tree.MediaContainer) {

            if (!server.synced) { //this is only for real servers

                if (tree.MediaContainer.Directory) {
                    $.each(tree.MediaContainer.Directory, function () {
                        i = Data.categories.length;
                        cat = {};
                        cat.id = i;
                        cat.name = (this["-title"] || "").replaceHtml();
                        cat.baseUrl = "/library/sections";
                        cat.url = this["-key"];
                        cat.type = this["-type"];
                        cat.art = this["-art"];
                        cat.server = id;
                        cat.serverName = server.name;
                        Data.categories.push(cat);
                    });
                }

                if (server.videoCount > 0) {
                    i = Data.categories.length;
                    cat = {};
                    cat.id = i;
                    cat.name = text.videochannels;
                    cat.baseUrl = "/video";
                    cat.url = "/channels/recentlyViewed?filter=video";
                    cat.type = "plugin";
                    cat.art = "";
                    cat.server = id;
                    cat.serverName = server.name;
                    Data.categories.push(cat);
                }

                if (server.musicCount > 0) {
                    i = Data.categories.length;
                    cat = {};
                    cat.id = i;
                    cat.name = text.musicchannels;
                    cat.baseUrl = "/music";
                    cat.url = "/channels/recentlyViewed?filter=music";
                    cat.type = "plugin";
                    cat.art = "";
                    cat.server = id;
                    cat.serverName = server.name;
                    Data.categories.push(cat);
                }

                if (server.photosCount > 0) {
                    i = Data.categories.length;
                    cat = {};
                    cat.id = i;
                    cat.name = text.photochannels;
                    cat.baseUrl = "/photos";
                    cat.url = "/channels/recentlyViewed?filter=photos";
                    cat.type = "plugin";
                    cat.art = "";
                    cat.server = id;
                    cat.serverName = server.name;
                    Data.categories.push(cat);
                }

                if (server.applicationsCount > 0) {
                    i = Data.categories.length;
                    cat = {};
                    cat.id = i;
                    cat.name = text.applications;
                    cat.baseUrl = "/applications";
                    cat.url = "/channels/recentlyViewed?filter=applications";
                    cat.type = "plugin";
                    cat.art = "";
                    cat.server = id;
                    cat.serverName = server.name;
                    Data.categories.push(cat);
                }

                i = Data.categories.length;
                cat = {};
                cat.id = i;
                cat.name = text.channeldir;
                cat.baseUrl = "";
                cat.url = "/system/plexonline";
                cat.type = "plexonline";
                cat.art = "/system/:/resources/store-art.png";
                cat.server = id;
                cat.serverName = server.name;
                Data.categories.push(cat);

            } else {
                //cloud sync
                if (tree.MediaContainer.Directory) {
                    $.each(tree.MediaContainer.Directory, function () {
                        cat = {};
                        cat.id = i;
                        cat.name = (this["-title"] || "").replaceHtml();
                        cat.baseUrl = "/library/sections";
                        cat.url = this["-key"];
                        cat.type = this["-type"];
                        cat.art = this["-art"];
                        cat.server = id;
                        cat.serverName = server.name;
                        Data.categories.push(cat);
                    });
                }
            }
        }

        Categorie.refresh();
    }
};

Categorie.loadDefaults = function () {

    var i;
    var cat;
    //add default categories
    i = Data.categories.length;
    cat = {};
    cat.id = i;
    cat.name = text.preferences;
    cat.baseUrl = "";
    cat.url = "preferences";
    cat.type = "preferences";
    cat.art = "Images/categorie/settings.png";
    cat.server = "";
    cat.serverName = "";
    Data.categories.push(cat);

    i = Data.categories.length;
    cat = {};
    cat.id = i;
    cat.name = text.quit;
    cat.baseUrl = "";
    cat.url = "quit";
    cat.type = "quit";
    cat.art = "Images/categorie/exit.png";
    cat.server = "";
    cat.serverName = "";
    Data.categories.push(cat);

    //if connected add myplex stuff
    if (MyPlex.loggedIn) {

        i = Data.categories.length;
        cat = {};
        cat.id = i;
        cat.name = text.queue;
        cat.baseUrl = "/pms/playlists/queue";
        cat.url = "https://plex.tv";
        cat.type = "queue";
        cat.art = "Images/categorie/art-default.png";
        cat.server = "https://plex.tv";
        cat.serverName = "Plex Account";
        Data.categories.push(cat);

        i = Data.categories.length;
        cat = {};
        cat.id = i;
        cat.name = text.recommended;
        cat.baseUrl = "/pms/playlists/recommendations";
        cat.url = "https://plex.tv";
        cat.type = "recommendations";
        cat.art = "Images/categorie/art-default.png";
        cat.server = "https://plex.tv";
        cat.serverName = "Plex Account";
        Data.categories.push(cat);

        i = Data.categories.length;
        cat = {};
        cat.id = i;
        cat.name = text.sharedContent;
        cat.baseUrl = "/library/sections";
        cat.url = "https://plex.tv/pms/system";
        cat.type = "shared";
        cat.art = "Images/categorie/art-default.png";
        cat.server = "https://plex.tv/pms/system";
        cat.serverName = "Plex Account";
        Data.categories.push(cat);
    }
};

Categorie.removeServer = function (server) {
    var filtered = _.filter(Data.category, function (cat) {
        return cat.server != server;
    });
    Data.category = [];
    Data.category = filtered;
    //redraw
    Display.setCategorieList();
};

Categorie.refresh = function () {
    var sorted = _.sortBy(Data.categories, function (cat) {
        return [cat["name"], cat["serverName"].join("_")];
    });

    Data.category = [];
    Data.category = sorted;

    while (Data.category.length < 11) {
        for (var k = 0; k < 2; k++) {
            var x = Data.category.length;
            for (var i = 0; i < x; i++) {
                var cat = {};
                cat.id = Data.category[i].id;
                cat.name = Data.category[i].name;
                cat.baseUrl = Data.category[i].baseUrl;
                cat.url = Data.category[i].url;
                cat.type = Data.category[i].type;
                cat.art = Data.category[i].art;
                cat.server = Data.category[i].server;
                cat.serverName = Data.category[i].serverName;
                Data.category.push(cat);
            }
            if (Data.category.length > 10)
                break;
        }
    }

    //pick the previous selected category
    for (var j = 0; j < Data.category.length; j++) {
        if (Data.category[j].id == Main.selectedCatID) {
            Main.selectedCat = j;
            break;
        }
    }

    //redraw
    Display.setCategorieList();
};


Categorie.handleError = function (xhrObj) {
    var serverCount = (prefs.servers.length || 0);
    var myPlexConn = 0;

    if (((prefs.myplex.autoLogin || "off") === "on") && ((prefs.myplex.authToken || "") !== "")) //myPlex 
    {
        myPlexConn = 1;
    }

    if (xhrObj) {
        if (xhrObj.status === 401) {
            Main.showInfo(text.errorPMS401);
            Main.debug("[Categorie.js] - Couldn't connect to PMS, Status: Unauthorized");
        }
        else if (serverCount > 0 && myPlexConn === 0) {
            Main.showInfo(text.errorPMS);
            Main.debug("[Categorie.js] - Couldn't connect to PMS");
        }
        else if (serverCount <= 0 && myPlexConn === 0) {
            Main.showInfo(text.errorAddPMS, "info", text.close);
            Main.debug("[Categorie.js] - No PMS defined and no Plex Account connection available");
        }
    } else {
        if (serverCount > 0 && myPlexConn === 0) {
            Main.showInfo(text.errorTimeout);
            Main.debug("[Categorie.js] - Couldn't connect to PMS");
        }
        else if (serverCount <= 0 && myPlexConn === 0) {
            Main.showInfo(text.errorAddPMS, "info", text.close);
            Main.debug("[Categorie.js] - No PMS defined and no Plex Account connection available");
        }
    }

    Main.error = 0;

};
