var Metadata =
{
    partId: 0,
    callback: null
};

var AudioStreams = [];
var SubtitleStreams = [];
var VideoStreams = [];
var MetaDataMedias = [];

Metadata.init = function () {
    VideoStreams = [];
    AudioStreams = [];
    SubtitleStreams = [];
    MetaDataMedias = [];
};

Metadata.fetchList = function (url, partid, callback) {

    if (partid) {
        Main.debug("[Metadata.js] - PartID: [" + partid + "]");
        Metadata.partId = partid;
    } else {
        Main.debug("[Metadata.js] - PartID is empty!");
        Metadata.partId = 0;
    }

    if (callback) {
        Metadata.callback = callback;
    }

    if (url) {
        var metaUrl = Main.server.resolveUrl(url, false, true);
        Main.debug("[Metadata.js] - Metadata URL: [" + metaUrl + "]");

        AjaxManager.parseHTTP(metaUrl, Metadata.streamloadCallback, null, 0, Content.id);
    }
    else {
        if (Metadata.callback) {
            Metadata.callback();
        }
    }
};

Metadata.streamloadCallback = function (tree, xhrObj, id) {

    if (xhrObj === null) {//timeout occured
        Main.debug("[Metadata.js] - Timeout occured, No streams loaded");
        if (Metadata.callback) {
            Metadata.callback();
        }
        return false;
    }

    if (xhrObj.status !== 200) {
        Main.debug("[Metadata.js] - Status returned: " + xhrObj.status + ", No streams loaded");
        if (Metadata.callback) {
            Metadata.callback();
        }
    }
    else {

        if (tree) {

            Metadata.init();

            $.each(tree.MediaContainer.Video[0].Media, function () {
                var media = $.extend(true, {}, this);
                MetaDataMedias.push(media);
            });

            $.each(tree.MediaContainer.Video[0].Media[Main.selectedMedia].Part[Player.selectedPart].Stream, function () {
                var streamType = parseInt((this["-streamType"] || "0"), 10);
                switch (streamType) {
                    case 1: //video
                        var videoStream = {
                            id: 0,
                            streamType: 1,
                            codec: '',
                            index: 0,
                            anamorphic: 0,
                            pixelAR: 0,
                            language: '',
                            languageCode: ''
                        };
                        videoStream.id = (this["-id"] || "");
                        videoStream.codec = (this["-codec"] || "");
                        videoStream.index = Number(this["-index"] || "0");
                        videoStream.anamorphic = Number(this["-anamorphic"] || "0");
                        videoStream.pixelAR = (this["-pixelAspectRatio"] || "");
                        videoStream.language = (this["-language"] || "");
                        videoStream.languageCode = (this["-languageCode"] || "");

                        VideoStreams.push(videoStream);

                        Main.debug("[Metadata.js] - Video Stream - Codec: [" + videoStream.codec + "] - Language: [" + videoStream.language +
                            "] - Stream ID: [" + videoStream.id + "] - Anamorphic: [" + (videoStream.anamorphic ? videoStream.anamorphic : "Not Available") +
                            "]- PixelAspectRatio: [" + (videoStream.pixelAR ? videoStream.pixelAR : "Not Available") + "]");

                        break;
                    case 2: //audio
                        var audioStream = {
                            id: 0,
                            streamType: 2,
                            codec: '',
                            channels: 2,
                            language: '',
                            languageCode: '',
                            selected: 0
                        };

                        audioStream.id = (this["-id"] || "");
                        audioStream.codec = (this["-codec"] || "");
                        audioStream.channels = Number(this["-channels"] || "2");
                        audioStream.language = (this["-language"] || "");
                        audioStream.languageCode = (this["-languageCode"] || "");
                        audioStream.selected = Number(this["-selected"] || "0");

                        AudioStreams.push(audioStream);

                        Main.debug("[Metadata.js] - Audio Stream - Codec: [" + audioStream.codec + "] - Language: [" + audioStream.language +
                            "] - Stream ID: [" + audioStream.id + "] - Channels: [" + audioStream.channels +
                            "] " + (audioStream.selected ? " <-- Selected" : ""));
                        break;
                    case 3: //subtitle
                        var subtitleStream = {
                            id: '',
                            streamType: 3,
                            key: '',
                            codec: '',
                            format: '',
                            language: '',
                            languageCode: '',
                            embedded: 0,
                            selected: 0
                        };
                        var strKey = (this["-key"] || "/library/metadata/" + this["-id"] || "");

                        if (strKey !== "") { //&& (strCodec === "srt" || strCodec === "smi")) {
                            subtitleStream.id = (this["-id"] || "");
                            subtitleStream.key = strKey;
                            subtitleStream.codec = (this["-codec"] || this["-format"] || "");
                            subtitleStream.format = (this["-format"] || this["-codec"] || "");
                            subtitleStream.language = (this["-language"] || "");
                            subtitleStream.languageCode = (this["-languageCode"] || "");
                            subtitleStream.embedded = (this["-key"]) ? 0 : 1; //subs without a key are embedded ones
                            subtitleStream.selected = Number(this["-selected"] || "0");

                            SubtitleStreams.push(subtitleStream);

                            Main.debug("[Metadata.js] - Subtitle Stream - Format: [" + subtitleStream.format + "] - Language: [" + subtitleStream.language +
                                "] - Stream ID: [" + subtitleStream.id + "] - Internal: [" + (subtitleStream.embedded ? "Yes" : "No") +
                                "] " + (subtitleStream.selected ? " <-- Selected" : ""));
                        }
                        break;
                    default:
                        Main.debug("[Metadata.js] - Unknown streamType: " + streamType);
                }
            });

            Main.debug("[Metadata.js] - Loading Metadata finished");
            if (Metadata.callback) {
                Metadata.callback();
            }
        }
    }
};


//##########################################
//  Subtitles
//##########################################
Metadata.getSubtitleCount = function () {
    return Number(SubtitleStreams.length || 0);
};

Metadata.getSubtitleUrl = function (index) {
    if (SubtitleStreams[index]) {
        return SubtitleStreams[index].key;
    }
};

Metadata.getSelectedSubtitle = function () {
    if (SubtitleStreams.length === 0) {
        return -1;
    }
    for (var i = 0; i < SubtitleStreams.length; i++) {
        var sub = SubtitleStreams[i];
        if (sub.selected == 1) {
            return i;
        }
    }
    return 0;
};

Metadata.setSelectedSubtitle = function (index) {
    //set selected stream: /library/parts/XXX?audioStreamID=X&subtitleStreamID=Y
    Main.debug("[Metadata.js] - setSelectedSubtitle index " + index);

    if (SubtitleStreams.length > 0) {
        var partId;
        var x;
        var i;
        if (index > -1) {

            var streamIndex = 0;
            for (x = 0; x < MetaDataMedias[Main.selectedMedia].Part.length; x++) {
                partId = MetaDataMedias[Main.selectedMedia].Part[x]["-id"];
                for (var j = 0; j < MetaDataMedias[Main.selectedMedia].Part[x].Stream.length; j++) {
                    if (MetaDataMedias[Main.selectedMedia].Part[x].Stream[j]["-id"] == SubtitleStreams[index].id) {
                        streamIndex = j;
                        break;
                    }
                }
                var streamid = MetaDataMedias[Main.selectedMedia].Part[x].Stream[streamIndex]["-id"];
                Main.debug("[Metadata.js] - set subtitle index " + index + " for Part: " + partId + " to: " + streamid + " check: " + SubtitleStreams[index].id);
                Main.updatePMSValue(partId, "subtitleStreamID", streamid);
            }

            //update the list to reflect the change
            for (i = 0; i < SubtitleStreams.length; i++) {
                SubtitleStreams[i].selected = 0;
            }

            SubtitleStreams[index].selected = 1;
            Subtitle.subtitleIndex = index;
            Subtitle.init();

            if ($('#rightOSDSubtitleStreamData').length) {
                insertText("rightOSDSubtitleStreamData", (index + 1) + "/" + SubtitleStreams.length + " " + SubtitleStreams[index].language + " [" + SubtitleStreams[index].format.toUpperCase() + "]");
            }

        } else if (index === -1) {

            for (x = 0; x < MetaDataMedias[Main.selectedMedia].Part.length; x++) {
                partId = MetaDataMedias[Main.selectedMedia].Part[x]["-id"];
                Main.debug("[Metadata.js] - set subtitle index " + index + " for Part: " + partId + " to: none");
                Main.updatePMSValue(partId, "subtitleStreamID", "");
            }

            //update the list to reflect the change
            for (i = 0; i < SubtitleStreams.length; i++) {
                SubtitleStreams[i].selected = 0;
            }

            if ($('#rightOSDSubtitleStreamData').length) {
                insertText("rightOSDSubtitleStreamData", (index + 1) + "/" + SubtitleStreams.length + " " + text.none);
            }
        }
    } else {
        Main.debug("[Metadata.js] - Ignoring subtitle change, streams not init yet");
    }
};


//##########################################
//  Audio
//##########################################
Metadata.getAudioCount = function () {
    return (AudioStreams.length || 0);
};

Metadata.getAudioChannels = function (index) {
    if (AudioStreams[index]) {
        var result = AudioStreams[index].channels;
        if (result === 2)
            return "2.0";
        else if (result === 3)
            return "2.1";
        else if (result === 4)
            return "4.0";
        else if (result === 5)
            return "5.0";
        else if (result === 6)
            return "5.1";
        else if (result === 7)
            return "6.1";
        else if (result === 8)
            return "7.1";
        else
            return result;
    }
    return "";
};

Metadata.getAudioCodec = function (index) {
    if (AudioStreams[index]) {
        var codec = (AudioStreams[index].codec || "unknown").toLowerCase();
        var n = Number(AudioStreams[index].channels || 2);
        if ((codec === 'ac3' || codec === 'eac3' || codec === 'aac') && n > 2) {
            return "Dolby";
        } else if (codec === 'dca') {
            return "DTS ";
        } else if (codec === "dts" && n === 7) {
            return "DTS-ES " + n;
        } else {
            return "Stereo";
        }
    } else {
        return "Stereo";
    }
};

Metadata.getAudioLanguage = function (index) {
    if (AudioStreams[index]) {
        var result = AudioStreams[index].language;
        if (result === "?")
            return "Undefined";
        return result;
    }
};

Metadata.getAudioId = function (index) {
    if (AudioStreams[index]) {
        return AudioStreams[index].id;
    }
};

Metadata.getSelectedAudio = function () {
    if (AudioStreams.length === 0) {
        return -1;
    }

    for (var i = 0; i < AudioStreams.length - 1; i++) {
        if (AudioStreams[i].selected === 1) {
            return i;
        }
    }
    return 0;
};

Metadata.setSelectedAudio = function (index) {
    //set selected stream: /library/parts/XXX?audioStreamID=X&subtitleStreamID=Y
    Main.debug("[Metadata.js] - setSelectedAudio index " + index);

    if (AudioStreams.length > 0) {
        var partId;
        var x;
        var i;
        if (index > -1) {

            var streamIndex = 0;
            for (x = 0; x < MetaDataMedias[Main.selectedMedia].Part.length; x++) {
                partId = MetaDataMedias[Main.selectedMedia].Part[x]["-id"];
                for (var j = 0; j < MetaDataMedias[Main.selectedMedia].Part[x].Stream.length; j++) {
                    if (MetaDataMedias[Main.selectedMedia].Part[x].Stream[j]["-id"] == AudioStreams[index].id) {
                        streamIndex = j;
                        break;
                    }
                }
                var streamid = MetaDataMedias[Main.selectedMedia].Part[x].Stream[streamIndex]["-id"];
                Main.debug("[Metadata.js] - set audioStream index " + index + " for Part: " + partId + " to: " + streamid + " check: " + AudioStreams[index].id);
                Main.updatePMSValue(partId, "audioStreamID", streamid);
            }

            //update the list to reflect the change
            for (i = 0; i < AudioStreams.length; i++) {
                AudioStreams[i].selected = 0;
            }

            AudioStreams[index].selected = 1;
            Audio.audioIndex = index;

            if ($('#rightOSDAudioStreamData').length) {
                insertText("rightOSDAudioStreamData", (index + 1) + "/" + AudioStreams.length + " " + AudioStreams[index].language + " [" + AudioStreams[index].codec + "]");
            }

        } else if (index === -1) {

            for (x = 0; x < MetaDataMedias[Main.selectedMedia].Part.length; x++) {
                partId = MetaDataMedias[Main.selectedMedia].Part[x]["-id"];
                Main.debug("[Metadata.js] - set audioStream index " + index + " for Part: " + partId + " to: none");
                Main.updatePMSValue(partId, "audioStreamID", "");
            }

            //update the list to reflect the change
            for (i = 0; i < AudioStreams.length; i++) {
                AudioStreams[i].selected = 0;
            }

            Audio.audioIndex = 0;

            if ($('#rightOSDAudioStreamData').length) {
                insertText("rightOSDAudioStreamData", (index + 1) + "/" + AudioStreams.length + " " + text.none);
            }
        }
    } else {
        Main.debug("[Metadata.js] - Ignoring audioStream change, streams not init yet");
    }
};
