var Player =
{
    server: null,
    transcodeServer: null,
    plugin: null,
    sef: null,
    state: 0,
    intervalID: 0,
    timeoutID: 0,
    checkTimeoutID: 0,
    jumpToTimoutID: 0,
    jumpToTimoutActive: 0,
    jumpToTimeoutTime: 1000,
    helpVar: 0,
    jumpVar: 0,
    resumeTime: 0,
    aspectMode: 0,
    transcoding: 0,
    subtitleIDBeforeTranscode: 0,

    selectedPart: 0,
    selected: 0,
    screenMode: 0,
    stereoScopic: false,
    url: "",
    media: null,

    fastforward: 60,
    rewind: 60,
    jumpToStr: "",

    curTime: 0,
    totalTime: 0,
    wrongTotalTime: 0,
    curDuration: 0,
    jumpFromTime: 0,
    jumpToTime: 0,
    jumpDirection: "",
    jumpTimer: 0,

    subSize: 22,
    subColor: "white",
    subPos: 450,
    subOffset: 0,
    subColorArray: ["white", "black", "red", "green", "yellow", "orange", "grey"],
    selectedSubColor: 0,
    errorCount: 0,

    STOPPED: 0,
    PLAYING: 1,
    PAUSED: 2,
    FORWARD: 3,
    REWIND: 4,
    INPROGRESS: 5,
    ERROR: 6,
    INIT: 10,
    BUFFER: 99,

    SEARCHING: false,

    // BD-Player Front Display
    PLAY: 100,
    STOP: 101,
    PAUSE: 102,
    ONLINE: 200,

    ASPECT_DEFAULT: 0,
    ASPECT_STRETCH: 1,
    ASPECT_ZOOM: 2,

    autoplay: 0,
    OFF: 0,
    ON: 1,
    NOT: 2,

    playbackSpeeds: [-16, -8, -4, -2, 1, 2, 4, 8, 16],
    playbackSpeed: 0,
    playbackSpeedIdx: 4,

    MODE_VIDEO: 1,
    MODE_MUSIC: 2,
    MODE_CLIP: 3,
    mode: 0,

    albumImage: "",
    aristName: "",
    albumName: "",
    songName: "",
    songIndex: "",
    songProgress: "",
    themePlaying: 0,
    playDelayTimer: 0,

    contentTitle: "",
    contentYear: ""
};

var AutoPlayList = {};

/*
 ####################################
 #
 #       INIT/DEINIT
 #
 ####################################
 */

Player.init = function () {
    var success = true;
    Player.state = Player.STOPPED;
    Player.plugin = new Protoplayer();
    try {
        Player.plugin.Open();   //init the player , use Player.plugin.Open(true) to test 2010/old player
    }
    catch (err) {
        Player.plugin.Open(true);  //fallback to old version for 2010 emu
    }

    if (!Player.plugin) {
        success = false;
    }
    else {
        var mwPlugin = document.getElementById("pluginTVMW");

        if (!mwPlugin) {
            success = false;
        }
    }

    Main.debug("[Player.js] - Player Plugin initialized, using SEF: [" + Player.plugin.sef.toString() + "]");

    Protoplayer.prototype.OnCurrentPlayTimeHandler = 'Player.onCurrentPlayTimeHandler';
    Protoplayer.prototype.OnStreamInfoReadyHandler = 'Player.onStreamInfoReadyHandler';
    Protoplayer.prototype.OnBufferingStartHandler = 'Player.onBufferingStart';
    Protoplayer.prototype.OnBufferingProgressHandler = 'Player.onBufferingProgress';
    Protoplayer.prototype.OnBufferingCompleteHandler = 'Player.onBufferingComplete';
    Protoplayer.prototype.OnNetworkDisconnectedHandler = 'Player.onNetworkDisconnected';
    Protoplayer.prototype.OnRenderStartHandler = 'Player.onRenderStart';
    Protoplayer.prototype.OnRenderErrorHandler = 'Player.onRenderError';
    Protoplayer.prototype.OnConnectionFailedHandler = 'Player.onConnectionFailed';
    Protoplayer.prototype.OnStreamNotFoundHandler = 'Player.onStreamNotFound';
    Protoplayer.prototype.OnRenderingCompleteHandler = 'Player.onRenderingComplete';
    Protoplayer.prototype.OnAuthenticationFailedHandler = 'Player.onAuthenticationFailedHandler';

    return success;
};

Player.deinit = function () {
    Main.debug("[Player.js] - Player deinit !!! ");

    if (Player.plugin) {
        Player.plugin.Stop(); //close is not allowed on 2012
    }
    if (Player.transcoding === 1) {
        var objTrans = new Transcoder();
        objTrans.kill();
    }
};

/*
 ####################################
 #
 #       SET CONTENT
 #
 ####################################
 */

Player.setAutoPlayList = function (id) {
    Main.debug("[Player.js] - Player.setAutoPlayList id: " + id);
    try {
        //save the current server for the playlist, with music playback the user may browse to other servers.
        Player.server = new Server(Main.server.machineIdentifier);
        if (Player.server.transcoderVideo) {
            Player.transcodeServer = Player.server;
        } else {
            var mach = Player.server.getTranscodeServer("video", true);
            if (mach) {
                Player.transcodeServer = new Server(mach);
            } else {
                Player.transcodeServer = Player.server;
            }
        }

        Player.cookie = "";

        Main.debug("[Player.js] - Player Server set to: " + Player.server.name);
        Main.debug("[Player.js] - Player Transcode Server set to: " + Player.transcodeServer.name);

        //create a copy of the contentlist for autoplay
        var playList = JSON.stringify(Data, null, 4);
        Player.selected = parseInt(id, 10);

        AutoPlayList = new Object(); // clear the old list
        AutoPlayList = JSON.parse(playList);
        AutoPlayList.url = Main.currentUrl;
        Main.debug("[Player.js] - Player AutoPlayList contains " + AutoPlayList.contentKeys.length + " items");
        //Main.writeFile("PlayList.txt", playList); //for debugging only
        Player.songIndex = (Player.selected + 1) + "/" + AutoPlayList.contentKeys.length;
        if (Player.mode === Player.MODE_MUSIC) {
            Player.setAutoPlay(Player.ON);
        }
    } catch (e) {
        Main.debug("[Player.js] - Player.setAutoPlayList error: " + e.message);
    }
};

Player.setMedia = function (contentMedia) {
    try {
        Main.debug("[Player.js] - Player.setMedia");
        Player.media = null;
        Player.media = contentMedia; //JSON.parse(contentMedia);
        Player.url = "";
        Metadata.partId = 0;
        Player.aspectMode = Player.ASPECT_DEFAULT;
    } catch (e) {
    }
};

Player.setURL = function (url) {//only used directly for themes, use setMedia for all others!
    Player.url = url;
    Main.debug("[Player.js] - Player setURL: [" + Player.url + "]");
};

/*
 ####################################
 #
 #       SET CONTENT
 #
 ####################################
 */

Player.playMedia = function (time) {

    if (Player.mode === Player.MODE_MUSIC) {
        Main.debug("[Player.js] - Player.mode: MUSIC");
        if (Player.state === Player.PLAYING) {
            Player.stopMusic();
        }
    }
    else if (Player.mode === Player.MODE_CLIP) {
        Main.debug("[Player.js] - Player.mode: CLIP");
        if (Player.state !== Player.STOPPED) {
            Main.debug("[Player.js] - Play command ignored, player in incorrect state");
            return false;
        }
    }
    else {
        Player.mode = Player.MODE_VIDEO;
        Main.debug("[Player.js] - Player.mode: VIDEO");
        if (Player.state !== Player.STOPPED) {
            Main.debug("[Player.js] - Play command ignored, player in incorrect state");
            return false;
        }
    }

    Player.state = Player.INIT;

    if (time) {
        Player.resumeTime = Math.floor(time / 1000); // time from millisec to sec
    }
    else {
        Player.resumeTime = 0;
    }

    Main.debug("[Player.js] - Player.resumeTime: " + Player.resumeTime);
    Main.debug("[Player.js] - Main.selectedMedia: " + Main.selectedMedia);

    if (Player.media && Player.mode === Player.MODE_CLIP) {
        if (Player.media.length > 1 && (prefs.player.pickHighestResolution || "off") === "off") //we got more than one source, let the user pick one.
        {
            window.setTimeout(function () {
                Popup.showChoice(Player.media, "Media", text.multipleVersions, function (val) {
                    Main.debug("[Player.js] - Picked Media Choice: [" + val + "]");
                    if (val !== null) {
                        Main.selectedMedia = Number(val);
                        Player.selectPart();
                    } else {
                        Player.cancelPlayback();
                    }
                });
            }, 100);
        }
        else {
            Main.selectedMedia = 0;
            Player.selectPart();
        }
    }
    else {
        Main.selectedMedia = 0;
        Player.selectPart();
    }
};

Player.selectPart = function () {
    var x = Main.selectedMedia;
    if (Player.media) {
        if (Player.media[x].Part.length > 1 && Player.resumeTime === 0 && (prefs.player.pickFirstPart || "off") === "off") //we got more than one source, let the user pick one.
        {
            window.setTimeout(function () {
                Popup.showChoice(Player.media[x].Part, "Part", text.multipleParts, function (val) {
                    Main.debug("[Player.js] - Picked Part Choice: [" + val + "]");
                    if (val !== null) {
                        Player.selectedPart = Number(val);
                        Player.setPart();
                    } else {
                        Player.cancelPlayback();
                    }
                });
            }, 100);
        }
        else {
            Player.selectedPart = 0;
            Player.setPart();
        }
    }
    else {
        Player.selectedPart = 0;
        Player.setPart();
    }
};

Player.setPart = function () { //use Player.selectedPart to set the part to play
    var x = Main.selectedMedia;

    if (!Player.media) {
        if (Player.url !== "")  //it might be set by a bad behaving plugin with only video tags
        {
            Main.debug("[Player.js] - Media not set, falling back to url playback");

            if (Player.mode === Player.MODE_MUSIC) {
                loadbar.show();
                loading.hide();
            }
            else {
                Display.hide();
                loadbar.hide();
                loading.show();
            }

            window.setTimeout(Player.buildMediaUrl, 50, Player.url);
            return;
        }
        else {
            Main.debug("[Player.js] - Media not set, nothing to play");
            Display.show();
            return;
        }
    }

    if (Player.media[x].Part.length === 0) { //No parts available
        Main.debug("[Player.js] - No parts available, nothing to play");
        Display.show();
        return;
    }

    Main.debug("[Player.js] - Selected Part: " + (Player.selectedPart + 1) + " of " + Player.media[x].Part.length);

    if (Player.mode === Player.MODE_MUSIC) {
        loadbar.show();
        loading.hide();
    }
    else {
        Display.hide();
        loadbar.hide();
        loading.show();
    }

    if (Player.media[x].Part.length === 1) { //just one part available
        Player.selectedPart = 0;
        window.setTimeout(Player.loadStreams, 50);
    }
    else if (Player.media[x].Part.length > 1) { //more parts available, check which one to play

        if (Player.mode === Player.MODE_MUSIC) { //Start music always from the beginning, play the currently selected part
            window.setTimeout(Player.resolveIndirect, 50);
        }
        else {
            if (Player.resumeTime > 0) { // the media has been watched before, we might be in a different part.
                var restime = Player.resumeTime * 1000;
                var time = 0;
                var partDur = 0;
                for (var i = 0; i < Player.media[x].Part.length; i++) {
                    partDur = (Player.media) ? (Player.media[x].Part[i]["-duration"] || 0) : 0;
                    time = time + parseInt(partDur, 10);

                    if (time > restime) {  //check in which part we are
                        Player.selectedPart = i;
                        $("#osdContentTitle").text(Player.contentTitle + "  (" + text.part + " " + (Player.selectedPart + 1) + "/" + Player.media[x].Part.length + ")");
                        Main.debug("[Player.js] - Changed Selected Part to: " + (Player.selectedPart + 1) + " because of resumeTime: [" + restime + "]");
                        window.setTimeout(Player.loadStreams, 50);
                        break;
                    }
                    else {
                        var substract = parseInt(partDur, 10) / 1000;
                        var temp = parseInt(Player.resumeTime, 10);
                        Player.resumeTime = Math.round(temp - substract);
                    }
                }
            }
            else { // media was not watched before, start with what the user selected
                $("#osdContentTitle").text(Player.contentTitle + "  (" + text.part + " " + (Player.selectedPart + 1) + "/" + Player.media[x].Part.length + ")");
                window.setTimeout(Player.loadStreams, 50);
            }
        }
    }
};

Player.loadStreams = function () {
    //we know what part we are going to play, get the streams before we start playing
    try {
        Metadata.init(); //clear any stored streams

        if (Player.mode === Player.MODE_VIDEO) {
            if (Player.media) {
                if (Player.media[Main.selectedMedia].Part) {
                    Metadata.partId = Player.media[Main.selectedMedia].Part[Player.selectedPart]["-id"];
                }
            }

            var url = AutoPlayList.contentMetadataUrls[Player.selected];
            var key = AutoPlayList.contentRatingKeys[Player.selected];

            if (url !== "") {
                Metadata.fetchList(url, key, Player.setAudio);
            }
            else {
                Player.setAudio();
            }
        }
        else {
            Player.setAudio();
        }
    } catch (e) {
        Main.debug("[Player.js] - Player.loadStreams error: " + e.message, 1);
        Player.setAudio();
    }
};

Player.setAudio = function () {
    var codec = "";
    if (AudioStreams[Audio.audioIndex]) {
        codec = (AudioStreams[Audio.audioIndex].codec || Player.media[Main.selectedMedia]["-audioCodec"] || "");
    } else {
        codec = (Player.media) ? Player.media[Main.selectedMedia]["-audioCodec"] : "";
    }

    if (codec !== "") {
        Audio.setAudioOutput(codec); //use the preferred audio from the PMS
    }

    if (codec !== "" && Metadata.getAudioCount() === 0) {
        insertText("rightOSDAudioStreamData", "1/1 " + codec.toUpperCase());
    } else {
        insertText("rightOSDAudioStreamData", text.unknown);
    }
    Player.resolveIndirect();
};

Player.resolveIndirect = function () {
    var x = Main.selectedMedia;
    var y = Player.selectedPart;
    var postUrl = (Player.media[x]["-postURL"] || Player.media[x].Part[y]["-postURL"] || "");
    var indirect = (Player.media[x]["-indirect"] || Player.media[x].Part[y]["-indirect"] || "");
    var url = "";

    AutoPlayList.contentIdentifier = "";
    AutoPlayList.contentUserAgent = "";
    AutoPlayList.contentContent = "";
    AutoPlayList.contentHttpCookies = "";
    AutoPlayList.contentHttpHeaders = "";

    if (indirect !== "") {
        Main.debug("[Player.js] - indirect: " + indirect);
        if (postUrl !== "") {
            Main.debug("[Player.js] - postURL: " + postUrl);

            //1) make a HTTP request for the postURL from the client, and grab the response
            //2) send a POST request instead of a GET to PMS...
            //a) the body of the POST should be the full HTTP response received by the client (headers + body)
            //b) add the &postURL= argument to the URL you request, so the plug-in framework knows which URL the headers/body you're sending is for
            // use the public url to resolve: node.plexapp.com:32400/system/services/url/lookup?url=http://vimeo.com/22415883

            Main.debug("[Player.js] - postURL GET: " + url);
            AjaxManager.getHTTP(postUrl, Player.postUrlCallback, null, 0, Content.id, null);
        } else if (indirect !== "") {
            url = Player.transcodeServer.resolveUrl(AutoPlayList.contentMetadataUrls[Player.selected], false, true);
            Main.debug("[Player.js] - resolveIndirect Indirect url: " + url);
            AjaxManager.parseHTTP(url, Player.indirectUrlCallback, null, 0, Content.id, null);
        }
    } else {
        Player.buildMediaUrl();
    }
};

Player.postUrlCallback = function (tree, xhrObj, id) {
    try {
        if (xhrObj === null) { //timeout occured
            Content.handleError(null);
            Player.cancelPlayback();
            return false;
        }

        if (xhrObj.status !== 200) {
            Main.debug("[Player.js] - Player.postUrlCallback Error " + xhrObj.status);
            if (xhrObj.status !== 0) {
                Content.handleError(xhrObj);
                Player.cancelPlayback();
                return false;
            }
        }
        else {
            var data = xhrObj.getAllResponseHeaders() + '\r\n' + xhrObj.responseText;
            var postUrl = (Player.media[Main.selectedMedia]["-postURL"] || Player.media[Main.selectedMedia].Part[Player.selectedPart]["-postURL"] || "");
            var direct = (Player.media[Main.selectedMedia].Part[Player.selectedPart]["-key"]).replaceHtml();

            if (Player.transcodeServer.ip !== "plex.tv") { // replace the url with the local server url
                direct = direct.replace("http://node.plexapp.com:32400", "");
            }

            var url = Player.transcodeServer.resolveUrl(direct + '&postURL=' + encodeURIComponent(postUrl), false, false);

            Main.debug("[Player.js] - posting URL: " + url);
            AjaxManager.parseHTTP(url, Player.indirectUrlCallback, null, 0, Content.id, data);
        }
    } catch (e) {
        Main.debug("[Player.js] - Error occured in Player.postUrlCallback: " + e.message);
        Content.handleError();
        Player.cancelPlayback();
        return false;
    }
};

Player.indirectUrlCallback = function (tree, xhrObj, id) {
    var y = Player.selectedPart;
    var newUrl = "";
    var indirect = "";

    if (xhrObj === null) //timeout occured
    {
        Content.handleError();
        Player.cancelPlayback();
        return false;
    }

    if (xhrObj.status !== 200) {
        Main.debug("[Player.js] - Player.indirectUrlCallback Error " + xhrObj.status);
        if (xhrObj.status !== 0) {
            Content.handleError(xhrObj, id);
            Player.cancelPlayback();
            return false;
        }
    }
    else {
        if (tree) {
            //Main.debug("[Player.js] - Indirect: " + JSON.stringify(tree, null, 4));

            if (tree.MediaContainer) {

                AutoPlayList.contentIdentifier = (tree.MediaContainer["-identifier"] || "");
                AutoPlayList.contentUserAgent = (tree.MediaContainer["-userAgent"] || "");
                AutoPlayList.contentContent = (tree.MediaContainer["-content"] || "");
                AutoPlayList.contentHttpCookies = (tree.MediaContainer["-httpCookies"] || "");
                AutoPlayList.contentHttpHeaders = (tree.MediaContainer["-httpHeaders"] || "");

                if (tree.MediaContainer.Video) {
                    indirect = (tree.MediaContainer.Video[0].Media[0].Part) ?
                        (tree.MediaContainer.Video[0].Media[0].Part[y]["-indirect"] || tree.MediaContainer.Video[0].Media[0]["-indirect"] || tree.MediaContainer.Video[0]["-indirect"] || "") :
                        (tree.MediaContainer.Video[0].Media[0]["-indirect"] || tree.MediaContainer.Video[0]["-indirect"] || "");

                    newUrl = (tree.MediaContainer.Video[0].Media[0].Part) ?
                        (tree.MediaContainer.Video[0].Media[0].Part[y]["-key"] || tree.MediaContainer.Video[0].Media[0]["-key"] || tree.MediaContainer.Video[0]["-key"] || "").replaceHtml() :
                        (tree.MediaContainer.Video[0].Media[0]["-key"] || tree.MediaContainer.Video[0]["-key"]).replaceHtml();
                }
                else if (tree.MediaContainer.Track) {
                    newUrl = (tree.MediaContainer.Track[0].Part) ?
                        (tree.MediaContainer.Track[0].Part[0]["-key"] || tree.MediaContainer.Track[0]["-key"] || "").replaceHtml() :
                        (tree.MediaContainer.Track[0]["-key"] || "").replaceHtml();
                }

                if (indirect !== "") {  //call it again

                    var mach = Player.transcodeServer.getTranscodeServer("video", true);
                    var url = "";

                    if (mach === null) { //no server available
                        url = Player.transcodeServer.resolveUrl("http://node.plexapp.com:32400" + newUrl, false, false);
                    } else {
                        url = Player.transcodeServer.resolveUrl(newUrl, false, true, mach);
                    }

                    Main.debug("[Player.js] - indirectUrlCallback New Indirect Url: " + url);
                    AjaxManager.parseHTTP(url, Player.indirectUrlCallback, null, 0, Content.id);
                }
                else {
                    Main.debug("[Player.js] - indirectUrlCallback New target Url: " + newUrl);
                    Player.buildMediaUrl(newUrl);
                }
            }
            else {
                Content.handleError(xhrObj, id);
                keyLock.status = keyLock.NONE;
                Player.cancelPlayback();
            }
        }
    }
};

Player.buildMediaUrl = function (indirectUrl) {
    var mediaIndex = Main.selectedMedia || 0;
    var partIndex = Player.selectedPart || 0;
    var key = (AutoPlayList.contentMetadataUrls) ? AutoPlayList.contentMetadataUrls[Player.selected] : "";
    var ratingKey = (AutoPlayList.contentRatingKeys) ? AutoPlayList.contentRatingKeys[Player.selected] : "";
    var params = {};
    var useTranscode = false;
    var mediaUrl = "";
    var webkit = (Player.media) ? (Player.media[mediaIndex]["-webkit"] || "") : "";
    var protocol = (Player.media) ? (Player.media[mediaIndex]["-protocol"] || "") : "";
    var pbm = Main.getPlaybackMode();
    var audioText = Metadata.getAudioCodec(Audio.audioIndex) + " " + Metadata.getAudioChannels(Audio.audioIndex);

    Player.subtitleIDBeforeTranscode = Metadata.getSelectedSubtitle(); //save the current subtitle so we can restore it

    if (isEmpty(Player.media)) { //we only have an url
        useTranscode = false;
    } else {
        if (Player.mode === Player.MODE_VIDEO) {

            switch (pbm) {
                case 1:
                    //Auto Mode
                    Main.debug("[Player.js] - Auto Mode is enabled");
                    useTranscode = false;
                    prefs.player.directPlay = "on";
                    prefs.player.directStream = "on";

                    if (!Player.checkDirectPlay()) {
                        useTranscode = true;
                    }

                    break;
                case 2:
                    //Direct Play
                    Main.debug("[Player.js] - Direct play is enabled");
                    useTranscode = false;
                    prefs.player.directPlay = "on";
                    prefs.player.directStream = "off";
                    break;
                case 3:
                    //Direct Stream
                    Main.debug("[Player.js] - Direct play is enabled");
                    useTranscode = true;
                    prefs.player.directPlay = "off";
                    prefs.player.directStream = "on";
                    break;
                case 4:
                    //Transcoding
                    Main.debug("[Player.js] - Transcoding is enabled");
                    useTranscode = true;
                    prefs.player.directPlay = "off";
                    prefs.player.directStream = "off";
                    break;
                default:
                    Main.debug("[Main.js] - Unknown PlaybackMode: " + pbm);
            }
        } else {
            Main.debug("[Player.js] - Auto Mode is enabled");
            useTranscode = false;
            prefs.player.directPlay = "on";
            prefs.player.directStream = "on";

            if (!Player.checkDirectPlay()) {
                useTranscode = true;
            }
        }
    }

    if (indirectUrl) { //we got a new destination use that one
        mediaUrl = indirectUrl;
    } else { //use the original destination
        mediaUrl = (Player.media) ? (Player.media[mediaIndex].Part[partIndex]["-key"] || "") : "";
    }

    if (webkit !== "" || protocol === "webkit") {
        Main.debug("[Player.js] - Webkit detected, aborting playback");
        Player.cancelPlayback(text.errorWebkit);
        return false;
    } else {
        mediaUrl = Player.server.resolveUrl(mediaUrl, false, false);
    }

    if (AutoPlayList.contentHttpHeaders !== "") {
        params.httpHeaders = AutoPlayList.contentHttpHeaders;
        Main.debug("[Player.js] - HttpHeaders detected");
    }

    if (AutoPlayList.contentUserAgent !== "") {
        params.userAgent = AutoPlayList.contentUserAgent;
        Main.debug("[Player.js] - UserAgent detected");
    }

    if (AutoPlayList.contentHttpCookies !== "") {
        params.httpCookies = AutoPlayList.contentHttpCookies;
        Main.debug("[Player.js] - HttpCookies detected");
    }

    if (protocol === "hls") {
        useTranscode = false; //this is already in hls format
    }
    if (Main.getPlaybackMode() === 1) {
        if (useTranscode) {
            widgetAPI.putInnerHTML(document.getElementById("osdTranscoding"), text.transcode + " - " + audioText);
        } else {
            widgetAPI.putInnerHTML(document.getElementById("osdTranscoding"), text.preferencesDIRECTPLAY + " - " + audioText);
        }
    } else if (Main.getPlaybackMode() === 2) {
        widgetAPI.putInnerHTML(document.getElementById("osdTranscoding"), text.preferencesDIRECTPLAY + " - " + audioText);
    } else if (Main.getPlaybackMode() === 3) {
        widgetAPI.putInnerHTML(document.getElementById("osdTranscoding"), text.preferencesDIRECTSTREAM + " - " + audioText);
    } else if (Main.getPlaybackMode() === 4) {
        widgetAPI.putInnerHTML(document.getElementById("osdTranscoding"), text.transcode + " - " + audioText);
    }


    if (useTranscode === false) {
        //Main.debug("[Player.js] -  params: " + $.param(params, true));

        var directPlayUrl = Player.server.resolveUrl(mediaUrl + $.param(params), false, true);

        if (protocol === "hls") { //for channels like national geographic
            if (directPlayUrl.contains('?')) {
                Player.setURL(Player.server.resolveUrl(directPlayUrl, false, true) + "&|COMPONENT=HLS");
            } else {
                Player.setURL(Player.server.resolveUrl(directPlayUrl, false, true) + "?|COMPONENT=HLS");
            }
        } else {
            Player.setURL(Player.server.resolveUrl(directPlayUrl, false, true));
        }

        Main.debug("[Player.js] - Player Direct playing: " + Player.url);

        if (Player.mode === Player.MODE_MUSIC) {
            Player.playMusic();
        } else {
            Display.hide();
            Player.playPart();
        }

    } else {

        var objTrans = new Transcoder();
        Player.transcoding = 1;

        if (Player.mode === Player.MODE_MUSIC) {
            if (Player.server.transcoderAudio) {
                Main.debug("[Player.js] - Player starting music transcode: " + Player.url);
                Player.setURL(objTrans.music(AutoPlayList.contentIdentifier, mediaUrl, key));
                Player.playMusic();
            } else {
                Player.cancelPlayback(text.errorNoTranscoder);
            }
        } else {
            if (Player.server.transcoderVideo) {

                if (Main.getPlaybackMode() > 2 && (prefs.subtitles.preferOverlay || "off") == "on") {
                    if (Metadata.getSelectedSubtitle() > -1) {
                        Player.subtitleIDBeforeTranscode = SubtitleStreams[Metadata.getSelectedSubtitle()].id;
                        Main.debug("[Player.js] - SubtitleID Before Transcode: " + Player.subtitleIDBeforeTranscode);
                        var partid = (Player.media[Main.selectedMedia].Part[Player.selectedPart]["-id"] || "0");
                        Main.updatePMSValue(partid, "subtitleStreamID", "");
                    }
                }

                Main.debug("[Player.js] - Player starting video transcode: " + Player.url);
                objTrans.video(AutoPlayList.contentIdentifier, mediaUrl, key, params, mediaIndex, partIndex, ratingKey, Player.transcodeResponse);
            } else {
                Player.cancelPlayback(text.errorNoTranscoder);
            }
        }
    }
};

Player.transcodeResponse = function (tree, xhr, id) {
    //id contains the url that was returned by the transcoder
    window.setTimeout(function () {
        Main.debug("[Player.js] - transcode url response: " + id);

        if (id !== "") {
            if (id.contains('?')) {
                Player.setURL(id + "&|COMPONENT=HLS");
            } else {
                Player.setURL(id + "?|COMPONENT=HLS");
            }
            Player.playPart();
        } else {
            Player.cancelPlayback(text.errorNoTranscoder);
        }

    }, 1000);
};

Player.cancelPlayback = function (text) {
    loading.hide();
    Main.debug("[Player.js] - Player cancelPlayback: " + text);
    if (text) {
        Main.showInfo(text, "info");
    }
    Display.hideAudioPlayer();
    Display.show();
    Player.state = Player.STOPPED;
    Main.enableScreenSaver();
    return false;
};

Player.checkDirectPlay = function () {

    //plugins in this list require transcoding to work correctly
    if (AutoPlayList.contentIdentifier) {
        var transcodePlugins = ["com.plexapp.plugins.southpark", "com.plexapp.plugins.plexpodcast"];
        if (transcodePlugins.indexOf(AutoPlayList.contentIdentifier.toLowerCase()) > -1) {
            Main.debug("[Player.js] - [DirectPlay not possible, plugin forced transcoding for: " + AutoPlayList.contentIdentifier + "]");
            return false;
        }
    }

    var x = Main.selectedMedia;
    var y = Player.selectedPart;
    var audiocodec = (Player.media) ? (Player.media[x]["-audioCodec"] || Player.media[x].Part[y]["-audioCodec"] || "") : "";
    var videocodec = (Player.media) ? (Player.media[x]["-videoCodec"] || Player.media[x].Part[y]["-videoCodec"] || "") : "";
    var container = (Player.media) ? (Player.media[x]["-container"] || Player.media[x].Part[y]["-container"] || "") : "";

    var containerCheck = false;
    var audioCheck = false;
    var videoCheck = false;
    var videocodecs = "";
    var audiocodecs = "";

    if (Player.mode === Player.MODE_MUSIC) {
        $(profile.Client.DirectPlayProfiles.MusicProfile).each(function () {
            if (container === this["-container"]) {
                containerCheck = true;
                audiocodecs = this["-codec"];
                if (audiocodecs.contains(audiocodec)) {
                    audioCheck = true;
                }
            }
        });


    } else {
        $(profile.Client.DirectPlayProfiles.VideoProfile).each(function () {
            if (container === this["-container"]) {
                containerCheck = true;
                videocodecs = this["-codec"];
                audiocodecs = this["-audioCodec"];
                if (videocodecs.contains(videocodec)) {
                    videoCheck = true;
                }
                if (audiocodecs.contains(audiocodec)) {
                    audioCheck = true;
                }
            }
        });
    }

    Main.debug("[Player.js] - [DirectPlay profile check:]");
    var containerResult = (containerCheck) ? "OK" : "Unsupported";
    var videoResult = (videoCheck) ? "OK" : (videocodecs) ? "Unsupported , Supported codecs : " + videocodecs : "Unsupported";
    var audioResult = (audioCheck) ? "OK" : (audiocodecs) ? "Unsupported , Supported codecs : " + audiocodecs : "Unsupported";

    if (Player.mode === Player.MODE_MUSIC) {
        Main.debug("[Player.js] - [Container  : " + container + "] --> " + containerResult);
        Main.debug("[Player.js] - [Audio codec: " + audiocodec + "] --> " + audioResult);
        if (audioCheck && containerCheck) {
            Main.debug("[Player.js] - [DirectPlay possible]");
            return true;
        } else {
            Main.debug("[Player.js] - [DirectPlay not possible, falling back to transcoding]");
            return false;
        }
    } else {

        if ((audiocodec === "") && (videocodec === "") && (container === "")) {
            Main.debug("[Player.js] - [No information available, assuming that DirectPlay is possible]");
            return true;
        }

        Main.debug("[Player.js] - [Container  : " + container + "] --> " + containerResult);
        Main.debug("[Player.js] - [Video codec: " + videocodec + "] --> " + videoResult);
        Main.debug("[Player.js] - [Audio codec: " + audiocodec + "] --> " + audioResult);

        if (audioCheck && videoCheck && containerCheck) {
            Main.debug("[Player.js] - [DirectPlay possible]");
            return true;
        } else {
            Main.debug("[Player.js] - [DirectPlay not possible, falling back to transcoding]");
            return false;
        }
    }
};

/*
 ####################################
 #
 #       SCREEN
 #
 ####################################
 */

Player.setWindow = function () {
    var displayX = 0;
    var displayY = 0;
    var displayWidth = 960;
    var displayHeight = 540;
    var videoWidth = Player.plugin.GetVideoWidth();
    var videoHeight = Player.plugin.GetVideoHeight();
    var aspectRatio = videoWidth / videoHeight;

    if (videoWidth > 0) {
        Main.debug("[Player.js] - Player set Content Width: [" + videoWidth + "] - Height: [" + videoHeight + "] (from Samsung API)");
    }

    if (videoWidth === 0 && Player.media) {
        if (Player.media[Main.selectedMedia].Part) {
            videoWidth = parseInt((Player.media[Main.selectedMedia].Part[Player.selectedPart]["-width"] || 0), 10);
            videoHeight = parseInt((Player.media[Main.selectedMedia].Part[Player.selectedPart]["-height"] || 0), 10);
            aspectRatio = parseFloat((Player.media[Main.selectedMedia].Part[Player.selectedPart]["-aspectRatio"] || 0));
        }

        if (aspectRatio === 0) {
            videoWidth = parseInt((Player.media[Main.selectedMedia]["-width"] || 0), 10);
            videoHeight = parseInt((Player.media[Main.selectedMedia]["-height"] || 0), 10);
            aspectRatio = parseFloat((Player.media[Main.selectedMedia]["-aspectRatio"] || 0));
            Main.debug("[Player.js] - Player got Content Width: [" + videoWidth + "] - Height: [" + videoHeight + "] - Aspect Ratio: [" + aspectRatio + "] (from PMS Media)");
        }
        else {
            Main.debug("[Player.js] - Player got Content Width: [" + videoWidth + "] - Height: [" + videoHeight + "] - Aspect Ratio: [" + aspectRatio + "] (from PMS Part)");
        }
    }

    if (videoWidth === 0) { //get it from the AutoPlayList
        videoWidth = parseInt(AutoPlayList.contentWidths[Player.selected], 10);
        videoHeight = parseInt(AutoPlayList.contentHeights[Player.selected], 10);
        aspectRatio = parseFloat(AutoPlayList.contentAspectRatios[Player.selected] || videoWidth / videoHeight);
        Main.debug("[Player.js] - Player got Content Width: [" + videoWidth + "] - Height: [" + videoHeight + "] - Aspect Ratio: [" + aspectRatio + "] (from PMS Video)");
    }

    var result;
    if (videoWidth > 0 && videoHeight > 0 && aspectRatio > 0) {

        if (Metadata.partId !== 0) {
            if (VideoStreams[0]) {
                if (VideoStreams[0].anamorphic === 1) {
                    var pixelAR;
                    Main.debug("[Player.js] - Player found that Video is anamorphic - PixelAR: [" + VideoStreams[0].pixelAR + "]");
                    result = VideoStreams[0].pixelAR.split(":");
                    pixelAR = parseInt(result[0], 10) / parseInt(result[1], 10);
                    videoWidth = Math.round(videoWidth * pixelAR);
                    aspectRatio = videoWidth / videoHeight;
                    Main.debug("[Player.js] - Player got new Content Width: [" + videoWidth + "]");
                }
            }
        }

        if (aspectRatio >= 1.78) {
            var factorX = (960 / videoWidth);
            displayWidth = Math.min(960, Math.round(videoWidth * factorX));
            displayHeight = Math.min(540, Math.round(videoHeight * factorX));
        }
        else {
            var factorY = (540 / videoHeight);
            displayHeight = Math.min(540, Math.round(videoHeight * factorY));
            displayWidth = Math.min(960, Math.round(displayHeight * aspectRatio));
        }

        if (displayHeight > 530)
            displayHeight = 540;

        if (displayWidth > 950)
            displayWidth = 960;

        displayX = Math.floor((960 - displayWidth) / 2);
        displayY = Math.floor((540 - displayHeight) / 2);
    }
    else {
        Main.debug("[Player.js] - Player something is missing - Width: [" + videoWidth + "] - Height: [" + videoHeight + "] - Aspect Ratio: [" + aspectRatio + "] (from PMS)", 1);
    }

    result = Player.plugin.SetDisplayArea(displayX, displayY, displayWidth, displayHeight);// Set App Display Area

    Main.debug("[Player.js] - Player Set Video Display Area: [" + result + "] - PosX: [" + displayX + "] - PosY: [" + displayY + "] - Width: [" + displayWidth + "] - Height: [" + displayHeight + "]");
};

Player.setAspectRatio = function () {

    if (Player.aspectMode === Player.ASPECT_STRETCH) {
        Player.plugin.SetDisplayArea(0, 0, 960, 540);
        Main.debug("[Player.js] - Player set Aspect Mode to Stretch");
    } else {
        Player.aspectMode = Player.ASPECT_DEFAULT;
        Player.setWindow();
        Main.debug("[Player.js] - Player set Aspect Mode to Default");
    }

    Player.setAspectButtonText();

    if (Player.state === Player.PAUSED) {
        Player.state = Player.PLAYING;
        Player.pauseVideo();
    }

};

/*
 ####################################
 #
 #       PLAY
 #
 ####################################
 */

Player.playVideo = function (time) {
    Main.debug("[Player.js] - Player is SEF: [" + Player.plugin.sef.toString() + "]");
    //Main.debug("[Player.js] - Player FF: [" + Player.fastforward + "] - RW: [" + Player.rewind + "]");

    Player.setBuffer();

    if (Player.url === "") {
        Main.debug("[Player.js] - Player No videos to play");
    }
    else if (Player.state === Player.STOPPED) {
        Player.playMedia(time); //will callback to Player.playPart();
    }
    else if (Player.state === Player.PAUSED) {
        Player.frontPanelSetStatus(Player.PLAY);
        widgetAPI.putInnerHTML(document.getElementById("osdStatus"), "");
        widgetAPI.putInnerHTML(document.getElementById("osdStatusPic"), "<img src='Images/osd/play.png' />");
        Player.resumeVideo();
        Player.plugin.SetPlaybackSpeed(1);
        Player.playbackSpeed = 0;
        Player.playbackSpeedIdx = 4;
        if (Main.osd === 1)
            window.setTimeout("Main.slideOSD('down')", 2000);
    }
    else if (Player.state === Player.FORWARD || Player.state === Player.REWIND) {
        widgetAPI.putInnerHTML(document.getElementById("osdStatus"), "");
        widgetAPI.putInnerHTML(document.getElementById("osdStatusPic"), "<img src='Images/osd/play.png' />");
        Player.plugin.SetPlaybackSpeed(1);
        Player.playbackSpeed = 0;
        Player.playbackSpeedIdx = 4;
        Player.state = Player.PLAYING;
        if (Main.osd === 1)
            window.setTimeout("Main.slideOSD('down')", 2000);
    }
    else
        Main.debug("[Player.js] - Player Ignoring play key, not in correct state");
};

Player.playPart = function () { //this is callback from Player.setPart();

    //Main.debug("[Player.js] - Player.state: " + Player.state);

    if (Player.url === "") {
        Main.debug("[Player.js] - Player No content to play", 1);
    }
    else if (Player.state === Player.PLAYING) {

        if (Player.mode === Player.MODE_MUSIC) {
            Player.stopMusic();
            window.setTimeout(Player.playPart, 2000);
            return false;
        }
        else {
            Main.debug("[Player.js] - Player Content already playing");
            return false;
        }
    }
    else if (Player.state === Player.STOPPED || Player.state === Player.INIT) {

        Main.debug("[Player.js] - Player starting function playPart: " + Player.selectedPart);

        Player.contentTitle = (AutoPlayList.contentTitles) ? AutoPlayList.contentTitles[Player.selected] : "";
        Player.contentYear = (AutoPlayList.contentYears) ? AutoPlayList.contentYears[Player.selected] : "";
        Display.setOSDText(Player.selected);

        //set a small step to begin with, we will update it after we get a real duration
        Player.fastforward = 5;
        Player.rewind = 5;

        widgetAPI.putInnerHTML(document.getElementById("osdStatus"), "");
        widgetAPI.putInnerHTML(document.getElementById("osdStatusPic"), "<img src='Images/osd/play.png' />");

        Player.state = Player.BUFFER;
        keyLock.status = keyLock.PLAYERSTART;
        Player.curTime = 0;
        Player.wrongTotalTime = 0;

        if (Player.mode === Player.MODE_VIDEO) {
            Subtitle.init(Player.playContent); //this will callback to Player.playContent
        }
        else { //we don't care about subtitles and audio, just play the url
            Player.playContent();
        }
    }
    else if (Player.state === Player.PAUSED) {
        Player.pauseVideo();
    }
    else if (Player.state === Player.FORWARD || Player.state === Player.REWIND) {
        widgetAPI.putInnerHTML(document.getElementById("osdStatus"), "");
        widgetAPI.putInnerHTML(document.getElementById("osdStatusPic"), "<img src='Images/osd/play.png' />");
        Player.plugin.SetPlaybackSpeed(1);
        Player.playbackSpeed = 0;
        Player.playbackSpeedIdx = 4;
        Player.state = Player.PLAYING;
        if (Main.osd === 1)
            window.setTimeout("Main.slideOSD('down')", 2000);
    }
};

Player.playContent = function () { //this is a callback from Player.playPart()
    var retVal;
    var result = true;
    var md = "2D";

    Main.disableScreenSaver();

    Player.stereoScopic = false;
    Player.screenMode = Player.checkFileNameFor3D();

    if (Player.screenMode > 0) {
        result = Player.get3D(); // the file indicates 3D, lets see if we can play it
        if (result === true) {
            md = "3D"; // 3dmode is going to be set after the OnStreamReady Event
        }
    } else {
        result = true;
    }

    Main.debug("[Player.js] - Player get3D result: " + result.toString());

    if (result === true) {
        var time = Player.resumeTime;
        Main.debug("[Player.js] - Player starting function playContent");
        Player.jumpVar = 0;

        if (Player.url.contains("m3u8")) {
            if (!Player.server.supports(Player.server.features.universalTranscoder) || (prefs.player.universalTranscoder || "off") === "off") {
                time = 0; //we will use Player.delayedJump() to force it to jump
            }
            Main.debug("[Player.js] - Player HLS Streaming is enabled");
        }

        Main.debug("[Player.js] - Player.url: " + Player.url);

        Player.plugin.InitPlayer(Player.url);
        Player.plugin.SetDisplayArea(0, 0, 960, 540);
        Player.setWindow();
        Player.setBuffer();

        if (Player.plugin.sef === true && Player.screenMode > 0) {  //Only on 2011+ models
            Player.stereoScopic = Player.set3D(Player.screenMode);
        }


        if (time > 0) {
            retVal = Player.plugin.StartPlayback(time);
        } else {
            retVal = Player.plugin.StartPlayback();
        }

        if (retVal === 1) {
            Main.debug("[Player.js] - Player Starting " + md + " content: [" + retVal + "] - From: [" + time + "sec]");
            Player.frontPanelSetStatus(Player.PLAY);
        } else {
            Main.debug("[Player.js] - Player Couldn't start " + md + " content: [" + retVal + "]", 1);
            Player.stopVideo(Player.ERROR);
            Main.showInfo(text.errorFile);
        }

    }
    else {
        Player.stopVideo(Player.ERROR);
    }
};

Player.playMusic = function (theme) {
    if (Player.url === "") {
        Main.debug("[Player.js] - No theme to play");
    }
    else {
        if (Player.state === Player.PLAYING && Player.mode === Player.MODE_MUSIC) {
            Player.stopMusic();
        }

        //set small jumps
        Player.fastforward = 5;
        Player.rewind = 5;

        if (Player.state !== Player.PLAYING) {

            Display.setOSDText(Player.selected);

            Player.mode = Player.MODE_MUSIC;
            Player.state = Player.BUFFER;
            Player.plugin.SetDisplayArea(961, 541, 1, 1);
            Player.resumeTime = 0; //always start from the beginning

            if (Player.plugin.Play(Player.url)) {
                if (theme) {
                    Player.themePlaying = 1;
                }
                else {
                    Player.themePlaying = 0;
                    Display.showAudioPlayer();
                    Main.enableScreenSaver();// with music keep the SS enabled
                }
            }
        }
    }
};

Player.restartTranscodePlayback = function () {
    loading.show();
    var objTrans = new Transcoder();
    var tree = null;
    window.clearInterval(Player.intervalID);
    Player.resumeTime = Player.curTime / 1000;
    Player.plugin.Stop();
    Player.state = Player.STOPPED;
    objTrans.restart(function (tree, xhr, url) {
        Player.transcodeResponse(tree, xhr, url);
    });
};

/*
 ####################################
 #
 #       RESUME
 #
 ####################################
 */

Player.resumeVideo = function () {
    Player.state = Player.PLAYING;
    Main.disableScreenSaver();
    Player.plugin.Resume();
    Main.debug("[Player.js] - Player curTime after: [" + Player.curTime + "]");
};

/*
 ####################################
 #
 #       PAUSE
 #
 ####################################
 */

Player.pauseVideo = function (force) {
    if (Main.screenlock === 0) {
        if (Player.state === Player.PLAYING || Player.state === Player.FORWARD || Player.state === Player.REWIND || force) {
            Main.enableScreenSaver();
            Player.frontPanelSetStatus(Player.PAUSE);
            widgetAPI.putInnerHTML(document.getElementById("osdStatus"), "");
            widgetAPI.putInnerHTML(document.getElementById("osdStatusPic"), "<img src='Images/osd/pause.png' />");
            Player.state = Player.PAUSED;
            Player.plugin.Pause();
            if (Main.osd === 0 && Main.rightOSD === 0) // only show OSD if rightOSD is not visible
                Main.slideOSD("up");

            Main.debug("[Player.js] - Player was paused");
        }
        else if (Player.state === Player.PAUSED) {
            if (AudioStreams[Audio.audioIndex]) {
                var codec = (AudioStreams[Audio.audioIndex].codec || "");
                var channels = (AudioStreams[Audio.audioIndex].channels || "");
                if (codec !== "") {
                    codec = (Player.media) ? Player.media[Main.selectedMedia]["-audioCodec"] : "";
                    Audio.setAudioOutput(codec); //use the preferred audio from the PMS
                }
            }
            Player.frontPanelSetStatus(Player.PLAY);
            widgetAPI.putInnerHTML(document.getElementById("osdStatus"), "");
            widgetAPI.putInnerHTML(document.getElementById("osdStatusPic"), "<img src='Images/osd/play.png' />");
            Player.resumeVideo();
            Player.plugin.SetPlaybackSpeed(1);
            Player.playbackSpeed = 0;
            Player.playbackSpeedIdx = 4;
            if (Main.osd === 1)
                window.setTimeout("Main.slideOSD('down')", 2000);

            Main.debug("[Player.js] - Player resumed playback");
        }
        else
            Main.debug("[Player.js] - Player Ignoring pause key, not in correct state");
    }
    else
        Main.debug("[Player.js] - Player Ignoring pause key, popup is open");
};

/*
 ####################################
 #
 #       STOP
 #
 ####################################
 */

Player.stopVideo = function (jump) {

    try {

        if (Player.state !== Player.STOPPED) {

            keyLock.status = keyLock.BLOCKALL;

            // stop sending progress from content
            window.clearInterval(Player.intervalID);
            Player.intervalID = 0;
            window.clearTimeout(Player.timeoutID);
            Player.timeoutID = 0;

            buttonAll.timeoutID = 0;

            Subtitle.resetOnScreenSubtitle(); //Clear any subtitles that are onscreen

            if (Player.screenMode > 0) {
                if (getModel(2010) && jump !== Player.ERROR) { //only for 2010 models
                    Player.set3D(0);
                }
            }

            if (Player.themePlaying === 1)
                return;

            Player.plugin.Stop();
            Player.state = Player.STOPPED;
            Main.enableScreenSaver();

            if (Player.transcoding === 1) {
                var objTrans = new Transcoder();
                objTrans.kill();

                if (Player.mode === Player.MODE_VIDEO) {
                    if (Player.transcoding === 1) {//restore the selectedsubtitle, but check it, the user might have changed it
                        if (Player.subtitleIDBeforeTranscode !== Metadata.getSelectedSubtitle()) {
                            var partid = (Player.media[Main.selectedMedia].Part[Player.selectedPart]["-id"] || "0");
                            Main.updatePMSValue(partid, "subtitleStreamID", Player.subtitleIDBeforeTranscode);
                        }
                    }
                }
            }

            //reset OSDs
            if (Main.osd === 1) { // stopping Video but OSD is still there -> hide OSD
                Main.slideOSD("down");
            }
            if (Main.rightOSD === 1) { // stopping Video but OSD is still there -> hide OSD
                Main.slideRightOSD("right");
            }
            Main.resetRightOSD();

            //reset error timeout
            window.clearTimeout(Player.checkTimeoutID);
            Player.checkTimeoutID = 0;

            //reset local variables
            Player.resumeTime = 0;
            Player.jumpVar = 0;
            Player.errorCount = 0;
            Player.transcoding = 0;
            Player.playbackSpeed = 0;
            Player.playbackSpeedIdx = 4; // reset fast(forward/rewind) speedcounter
            Player.aspectMode = 0;
            Player.frontPanelSetStatus(Player.STOP);
            Player.themePlaying = 0;

            document.getElementById("playerStatus").style.display = "none";

            Display.hideAudioPlayer();

            if (jump === Player.INPROGRESS) {
                Main.debug("[Player.js] - Player stop playback INPROGRESS");

                var progressTime = 0;
                if (Player.curTime < 2003)
                    progressTime = Player.curTime;
                else
                    progressTime = Player.curTime - 2000;


                if (Player.selectedPart > 0) //add the durations of the previous parts
                {
                    var dur = 0;
                    for (var i = 0; i < Player.selectedPart; i++) {
                        var partDur = (Player.media) ? (Player.media[Main.selectedMedia].Part[i]["-duration"] || 0) : 0;
                        var tmp = parseInt(partDur, 10);
                        dur = dur + tmp;
                    }
                    progressTime += dur;
                    Main.debug("[Player.js] - Time compensated for extra parts: " + progressTime);
                }

                if (Player.mode === Player.MODE_VIDEO || Player.mode === Player.MODE_CLIP) {
                    if (AutoPlayList.contentRatingKeys[Player.selected] !== "" && progressTime > 500) {
                        Main.setProgress(
                            AutoPlayList.contentMetadataUrls[Player.selected],
                            AutoPlayList.contentRatingKeys[Player.selected],
                            progressTime,
                            Player.duration,
                            AutoPlayList.identifier,
                            Player.server,
                            "stopped",
                            Player.refreshDisplay);
                    }
                }

                Display.show();
            }
            else if (jump === Main.WATCHED) {
                Main.debug("[Player.js] - Player stop playback WATCHED");

                var partCount = 0;

                if (Player.media) {
                    if (Player.media[Main.selectedMedia].Part) {
                        partCount = Player.media[Main.selectedMedia].Part.length - 1;
                    }
                }

                if (Player.selectedPart < partCount) //there are more parts to be played
                {
                    Player.selectedPart++;
                    Player.setPart(); //play the next part
                }
                else {

                    if (Player.mode === Player.MODE_VIDEO || Player.mode === Player.MODE_CLIP) {
                        if (AutoPlayList.contentRatingKeys[Player.selected] !== "") {
                            Main.setProgress(
                                AutoPlayList.contentMetadataUrls[Player.selected],
                                AutoPlayList.contentRatingKeys[Player.selected],
                                0,
                                Player.duration,
                                AutoPlayList.identifier,
                                Player.server,
                                "stopped",
                                function () {
                                    Main.setWatched(AutoPlayList.contentRatingKeys[Player.selected], Main.WATCHED, AutoPlayList.identifier, Player.server, function () {

                                        if (Player.autoplay === Player.ON) {
                                            if (AutoPlayList.contentMedias.length > Player.selected) { //we have more media to play
                                                window.setTimeout(Player.setAutoPlayNext, 500, Main.NEXT);
                                            }
                                        } else {
                                            Player.refreshDisplay();
                                        }

                                    });
                                });
                        }
                    } else {
                        if (Player.autoplay === Player.ON) {
                            if (AutoPlayList.contentMedias.length > Player.selected) { //we have more media to play
                                window.setTimeout(Player.setAutoPlayNext, 500, Main.NEXT);
                            }
                        } else {
                            Display.show();
                        }
                    }
                }
            }
            else if (jump === Player.ERROR) {
                Main.debug("[Player.js] - Player stop playback ERROR", 1);
                if (Player.state === Player.STOPPED) {
                    Display.show();
                }
            }
            else if (jump === Main.NETWORKERROR) {
                Main.debug("[Player.js] - Player stop playback NETWORKERROR", 1);
                if (Player.autoplay === Player.ON) {
                    Player.autoplay = Player.OFF;
                    widgetAPI.putInnerHTML(document.getElementById("bottomAutoplayText"), text.playerAutoplayOff);
                    widgetAPI.putInnerHTML(document.getElementById("osdAutoplayText"), text.OSDAutoplayOff);
                }
                Display.show();
            }
        }
        else {
            loading.hide(); // always do this player could be buffering
            loadbar.hide();
            Main.debug("[Player.js] - Player Ignoring stop request, not in correct state");
        }
    } catch (e) {
        Main.debug("[Player.js] -  Player stop failed: " + e.message);
        Display.show();
    }
};

Player.refreshDisplay = function () {
    if (Main.view === Main.CATEGORIE) {
        widgetAPI.putInnerHTML(document.getElementById("bottomEnterText"), text.categoryBottomEnterCat);
        widgetAPI.putInnerHTML(document.getElementById("bottomReturnText"), text.categoryBottomExitApp);
        buttonCategorie.showCategories();
        buttonCategorie.updateCurrentCat();
    } else if (Main.view === Main.MOVIE) {
        buttonMovie.refreshList();
    } else if (Main.view === Main.EPISODE) {
        buttonEpisode.refreshList();
    } else if (Main.view === Main.SECTIONS) {
        buttonCatSection.refreshList();
    }
    Display.show();
};

Player.stopMusic = function () {
    if (Player.state !== Player.STOPPED) {
        Player.plugin.Stop();
        Main.enableScreenSaver();
        if (Player.transcoding === 1) {
            var objTrans = new Transcoder();
            objTrans.kill();
        }
        Player.themePlaying = 0;
        Display.hideAudioPlayer();
        keyLock.status = keyLock.NONE;
        Player.state = Player.STOPPED;
        Main.debug("[Player.js] - Music stopped");
    }
    else {
        Main.debug("[Player.js] - Ignoring stop request, not in correct state");
    }
};

/*
 ###############################
 #
 #       AUTOPLAY
 #
 ###############################
 */

Player.setAutoPlay = function (state) {

    if (state === Player.ON || state === Player.OFF) {//only set if OFF or ON
        Player.autoplay = state;
    }

    if (Player.autoplay === Player.ON) {
        Main.debug("[Player.js] - setAutoPlay: ON");
        $("#osdAutoplayText").text(text.OSDAutoplayOn).show();
        $("#osdAutoplayPic").show();
        $("#bottomAutoplayPic").show();
        $("#bottomAutoplay").show();
        $("#bottomAutoplay").find("img").attr('src', 'Images/navi/autoplay.png');
    }
    else if (Player.autoplay === Player.OFF) {
        Main.debug("[Player.js] - setAutoPlay: OFF");
        $("#osdAutoplayText").text(text.OSDAutoplayOff).show();
        $("#osdAutoplayPic").show();
        $("#bottomAutoplayPic").show();
        $("#bottomAutoplay").show();
        $("#bottomAutoplay").find("img").attr('src', 'Images/navi/autoplay-off.png');
    }
    else if (Player.autoplay === Player.NOT) {
        $("#osdAutoplayText").hide();
        $("#osdAutoplayPic").hide();
        $("#bottomAutoplayPic").hide();
        $("#bottomAutoplay").hide();
    }
};

Player.setAutoPlayNext = function (direction, manual) {

    Player.plugin.Stop(); //make sure the player stopped
    Player.state = Player.STOPPED;
    Main.enableScreenSaver();
    if (Player.transcoding === 1) {
        var objTrans = new Transcoder();
        objTrans.kill();
    }

    window.clearTimeout(Player.playDelayTimer);

    if (!manual) {
        if (Player.autoplay === Player.OFF) {
            return false; //the user canceled autoplay
        }
    }

    if (direction === Main.NEXT) {
        if ((Player.selected + 1) > AutoPlayList.contentKeys.length - 1) {
            Player.selected = 0;
        }
        else { //we have more media to play
            Player.selected++;
        }
    }
    else if (direction === Main.PREV) {
        if ((Player.selected - 1) < 0) {
            Player.selected = AutoPlayList.contentKeys.length - 1;
        } else {
            Player.selected--;
        }
    }

    Main.debug("[Player.js] - Player.setAutoPlayNext: " + (Player.selected + 1) + " of " + AutoPlayList.contentKeys.length);

    Player.media = null;
    Player.url = "";
    Metadata.partId = 0;
    Player.selectedPart = 0;

    //Main.debug("[Player.js] - AutoPlayList.contentMedias[Player.selected]: " + AutoPlayList.contentMedias[Player.selected]);

    if (AutoPlayList.contentContainerType[Player.selected] === "Video") {
        if (Main.mediaContent === Main.MOVIE || Main.mediaContent === Main.EPISODE || Main.mediaContent === Main.SEASON || Main.mediaContent === Main.SHOW) {
            Player.mode = Player.MODE_VIDEO;
        } else {
            Player.mode = Player.MODE_CLIP;
        }
    } else if (AutoPlayList.contentContainerType[Player.selected] === "Track") {
        Player.mode = Player.MODE_MUSIC;

        //var pic = Player.server.resolveUrl(AutoPlayList.contentThumbs[Player.selected], false, true);

        Player.albumImage = Main.getPhoto(AutoPlayList.contentThumbs[Player.selected], 70, 70, "artist");
        Player.aristName = AutoPlayList.contentGrandParentTitles[Player.selected];
        Player.albumName = AutoPlayList.contentParentTitles[Player.selected];
        Player.songName = AutoPlayList.contentTitles[Player.selected];
        Player.contentTitle = AutoPlayList.contentTitles[Player.selected];
        Player.contentYear = Player.aristName + ", " + Player.albumName;
        Player.songIndex = (Player.selected + 1) + "/" + AutoPlayList.contentKeys.length;

        Main.debug("[Player.js] - Player.setAutoPlayNext: " + Player.songName);

        $("#albumImage").find("img").attr('src', Player.albumImage).attr('width', 70).attr('height', 70);
        $("#artistName").text(Player.aristName);
        $("#albumName").text(Player.albumName);
        $("#songName").text(Player.songName);
        $("#songProgress").text(text.stopped);
        $("#songIndex").text(Player.songIndex);

        Display.show();
    }

    if (AutoPlayList.contentMedias[Player.selected] !== "") { //we have more media to play
        Player.media = AutoPlayList.contentMedias[Player.selected];
    } else if (AutoPlayList.contentKeys[Player.selected] !== "") {//this item has no media tag, use the url instead
        if (AutoPlayList.contentKeys[Player.selected].startsWith("/")) {
            Player.setURL(Player.server.resolveUrl(AutoPlayList.contentKeys[Player.selected], false, true));
        } else {
            var prefix = AutoPlayList.url.leftFrom("?");
            Player.setURL(Player.server.resolveUrl(prefix + "/" + AutoPlayList.contentKeys[Player.selected], false, true));
        }
    } else { //nothing to play
        Display.show();
        return false;
    }

    $("#osdContentTitle").text(Player.contentTitle + " AutoPlaying: " + (Player.selected + 1) + " of " + AutoPlayList.contentKeys.length);
    $("#osdContentYear").text(Player.contentYear);

    Player.playDelayTimer = window.setTimeout(Player.playMedia, 500); //always start at the beginning when autoplaying, so don't add the resumetime
};

/*
 ###############################
 #
 #       PLAYBACK SPEED
 #
 ###############################
 */

Player.setPlaybackSpeed = function (direction) {
    Player.playbackSpeed = Player.playbackSpeeds[Player.playbackSpeedIdx];
    var result = Player.plugin.SetPlaybackSpeed(Player.playbackSpeed);

    Main.debug("[Player.js] - set playback speed: " + Player.playbackSpeed + " result: " + result);

    if (result === 1) {
        if (Player.playbackSpeed === 1) {
            Player.state = Player.PLAYING;
            widgetAPI.putInnerHTML(document.getElementById("osdStatus"), "");
            widgetAPI.putInnerHTML(document.getElementById("osdStatusPic"), "<img src='Images/osd/play.png' />");
            window.setTimeout("Main.slideOSD('down')", 2000);
        }
        else if (Player.playbackSpeed > 1) {
            Player.state = Player.FORWARD;
            widgetAPI.putInnerHTML(document.getElementById("srtSubtitle"), "");
            widgetAPI.putInnerHTML(document.getElementById("osdStatus"), Player.playbackSpeed + "x");
            widgetAPI.putInnerHTML(document.getElementById("osdStatusPic"), "<img src='Images/osd/ff.png' />");
        }
        else if (Player.playbackSpeed < 1) {
            Player.state = Player.REWIND;
            widgetAPI.putInnerHTML(document.getElementById("srtSubtitle"), "");
            widgetAPI.putInnerHTML(document.getElementById("osdStatus"), Math.abs(Player.playbackSpeed) + "x");
            widgetAPI.putInnerHTML(document.getElementById("osdStatusPic"), "<img src='Images/osd/rw.png' />");
        }
    }
    else {
        window.setTimeout("Main.slideOSD('down')", 2000);
        if (direction === "FF") {
            Main.showInfo(text.errorFF, "info", text.continue1);
        }
        else {
            Main.showInfo(text.errorFR, "info", text.continue1);
        }
        Player.playbackSpeed = 0; //reset to normal speed
        Player.playbackSpeedIdx = 4;
        Player.plugin.SetPlaybackSpeed(1);
        Player.state = Player.PLAYING;
    }
};

/*
 ###############################
 #
 #       FASTFORWARD
 #
 ###############################
 */

Player.fastForwardVideo = function () {
    Player.playbackSpeedIdx++;
    if (Player.playbackSpeedIdx > 8) {
        Player.playbackSpeedIdx = 8;
        Main.debug("[Player.js] - FF already at max speed");
        return;
    }
    Player.setPlaybackSpeed("FF");
};

/*
 ###############################
 #
 #       FASTREWIND
 #
 ###############################
 */

Player.fastBackwardVideo = function () {
    Player.playbackSpeedIdx--;
    if (Player.playbackSpeedIdx < 0) {
        Player.playbackSpeedIdx = 0;
        Main.debug("[Player.js] - RW already at max speed");
        return;
    }
    Player.setPlaybackSpeed("RW");
};

/*
 ###############################
 #
 #       JUMP FORWARD
 #
 ###############################
 */

Player.skipForwardVideo = function (number) {
    if (Main.screenlock === 0 || Player.jumpDirection !== "") {
        Player.jumpDirection = "FF";
        Player.jumpFromTime = parseInt(Player.curTime / 1000, 10);

        if (!number)
            number = Player.fastforward;

        Player.SEARCHING = true;

        if (Main.osd === 0) {
            Main.slideOSD("up");
            Player.jumpToTimeoutTime = 2000;
        }
        else {
            window.clearTimeout(Player.timeoutID); //stop the osd from hiding itself
        }

        if (Player.jumpToTimoutActive === 1) {
            window.clearTimeout(Player.jumpToTimoutID); //cancel previous request
            Player.jumpToTime = Player.jumpToTime + parseInt(number, 10); // add the time, user is still pushing the button
        }
        else {
            Player.jumpToTime = parseInt(number, 10);
        }

        if ((parseInt(Player.curTime / 1000, 10) + Player.jumpToTime) > parseInt(Player.totalTime / 1000, 10)) {

            Main.debug("[Player.js] - Player.jumpFromTime + Player.jumpToTime too long");
            Main.debug("[Player.js] - Player.jumpFromTime: " + Player.jumpFromTime);
            Main.debug("[Player.js] - Player.jumpToTime: " + Player.jumpToTime);
            Main.debug("[Player.js] - Player.totalTime: " + parseInt(Player.totalTime, 10) / 1000);
            Player.jumpToTime = parseInt((Player.totalTime) / 1000, 10) - parseInt(Player.curTime / 1000, 10) - 10; // we need 10 secs buffer else the jump fails
            Main.debug("[Player.js] - New time : " + Player.jumpToTime);

        }

        Player.jumpToTimoutID = window.setTimeout(Player.doForwardJump, Player.jumpToTimeoutTime, Player.jumpToTime);
        Player.jumpToTimeoutTime = 1000;
        Player.jumpToTimoutActive = 1;

        var timePercent = (100 * (Player.jumpFromTime + Player.jumpToTime)) / parseInt(Player.totalTime / 1000, 10);
        var breit = 886 / 100 * timePercent;

        breit = breit + 37;
        document.getElementById("progress_pic").style.width = timePercent + "%";
        document.getElementById("osdProgressBarRight").style.left = breit + "px";
    }
    else {
        Main.debug("[Player.js] - Player Ignoring JumpForward key, popup is open");
    }

};

Player.doForwardJump = function (time) {
    if (Player.curTime >= Player.wrongTotalTime && Player.wrongTotalTime !== 0) {
        Main.debug("[Player.js] - Player Ignoring Jump request because current Time is further than wrong Total Time so jump would stop video");
    }
    else {
        widgetAPI.putInnerHTML(document.getElementById("srtSubtitle"), ""); //hide subs while we jump

        var result = Player.plugin.JumpForward(time);

        if (result === 1) {
            window.clearInterval(Player.intervalID);
            loading.show();
        }

        Main.debug("[Player.js] - Player JumpForward: [" + time + "]" + " seconds,  result : [" + result + "]");

        Player.jumpToTimoutActive = 0;
        Player.jumpToTime = 0;
        Player.jumpDirection = "";
        Player.SEARCHING = false;

        if (result === 1 && Player.state === Player.PAUSED) {
            Player.pauseVideo(true);
        }

        if (getModel(2010) && result === 1) // have to setOffScreenSaver because C-Series starts the video automatically after it jumps
        {
            Player.state = Player.PAUSED;
            Player.pauseVideo();
        }
    }

    if (Main.osd === 1) {
        Player.timeoutID = window.setTimeout("Main.slideOSD('down')", 2500);
    }
};

/*
 ###############################
 #
 #       JUMP BACKWARDS
 #
 ###############################
 */

Player.skipBackwardVideo = function (number) {
    if (Main.screenlock === 0 || Player.jumpDirection !== "") {
        Player.jumpDirection = "RW";

        if (!number)
            number = Player.rewind;

        Player.SEARCHING = true;

        if (Main.osd === 0) {
            Main.slideOSD("up");
            Player.jumpToTimeoutTime = 2000;
        }
        else {
            window.clearTimeout(Player.timeoutID); //stop the osd from hiding itself
        }

        if (Player.jumpToTimoutActive === 1) {
            window.clearTimeout(Player.jumpToTimoutID); //cancel previous request
            Player.jumpToTime = Player.jumpToTime + parseInt(number, 10); // add the time, user is still pushing the button
        }
        else {
            Player.jumpToTime = parseInt(number, 10);
            Player.jumpFromTime = parseInt(Player.curTime / 1000, 10);
        }

        if ((parseInt(Player.curTime / 1000, 10) - Player.jumpToTime) <= 0) {
            Player.jumpToTime = parseInt(Player.curTime / 1000, 10);
        }

        Player.jumpToTimoutID = window.setTimeout(Player.doBackwardJump, Player.jumpToTimeoutTime, Player.jumpToTime);
        Player.jumpToTimeoutTime = 1000;
        Player.jumpToTimoutActive = 1;

        var timePercent = (100 * (parseInt(Player.curTime / 1000, 10) - Player.jumpToTime)) / parseInt(Player.totalTime / 1000, 10);
        var breit = 886 / 100 * timePercent;

        breit = breit + 37;
        document.getElementById("progress_pic").style.width = timePercent + "%";
        document.getElementById("osdProgressBarRight").style.left = breit + "px";
    }
    else {
        Main.debug("[Player.js] - Player Ignoring JumpForward key, popup is open");
    }

};

Player.doBackwardJump = function (time) {
    widgetAPI.putInnerHTML(document.getElementById("srtSubtitle"), ""); //hide subs while we jump
    var result = Player.plugin.JumpBackward(time);

    if (result === 1) {
        window.clearInterval(Player.intervalID);
        loading.show();
    }

    Main.debug("[Player.js] - Player JumpBackward: [" + time + "]" + " seconds,  result : [" + result + "]");

    Player.jumpToTimoutActive = 0;
    Player.jumpToTime = 0;
    Player.jumpDirection = "";
    Player.SEARCHING = false;

    if (Player.curTime === 0 && result === 1 && Player.state === Player.PAUSED) // if curTime is 0 video will automatically start playing
    {
        Player.pauseVideo(true);
    }

    if (result === 1 && Player.state === Player.PAUSED) // if curTime is 0 video will automatically start playing
    {
        Player.pauseVideo(true);
    }

    if (getModel(2010) && result === 1) // have to setOffScreenSaver because C-Series starts the video automatically after it jumps
    {
        Player.state = Player.PAUSED;
        Player.pauseVideo();
    }

    if (Main.osd === 1) {
        Player.timeoutID = window.setTimeout("Main.slideOSD('down')", 2500);
    }
};

/*
 ###############################
 #
 #       JUMP TO
 #
 ###############################
 */

Player.jumpTo = function (number) {
    var extSec = 0;
    var extMin = 0;
    var extHour = 0;
    var targetTime = 0;
    var targetTimeInMS = 0;

    if (Player.jumpToStr.length < 7 && Main.selectedRightOSD === 8 && Player.state !== Player.STOPPED && Main.rightOSD === 1) {
        if (number) {
            Player.jumpToStr = Player.jumpToStr + number;
            insertText("rightOSDBottomReturnText", text.rightOSDBottomReset);
        }
        if (Player.jumpToStr.length > 0 && Player.jumpToStr.length < 3) // writing secs
        {
            extSec = Player.jumpToStr.substr(0);
            $("#rightOSDJumpToData").text(extSec + "sec");
        }
        else if (Player.jumpToStr.length > 2 && Player.jumpToStr.length < 5) // writing mins
        {
            if (Player.jumpToStr.length === 3) {
                extSec = Player.jumpToStr.substr(1);
                extMin = Player.jumpToStr.substr(0, 1);
            }
            else if (Player.jumpToStr.length === 4) {
                extSec = Player.jumpToStr.substr(2);
                extMin = Player.jumpToStr.substr(0, 2);
            }
            $("#rightOSDJumpToData").text(extMin + "min " + extSec + "sec");
        }
        else if (Player.jumpToStr.length > 4 && Player.jumpToStr.length < 7) // writing hours
        {
            if (Player.jumpToStr.length === 5) {
                extSec = Player.jumpToStr.substr(3);
                extMin = Player.jumpToStr.substr(1, 2);
                extHour = Player.jumpToStr.substr(0, 1);
            }
            else if (Player.jumpToStr.length === 6) {
                extSec = Player.jumpToStr.substr(4, 2);
                extMin = Player.jumpToStr.substr(2, 2);
                extHour = Player.jumpToStr.substr(0, 2);
            }
            $("#rightOSDJumpToData").text(extHour + "h " + extMin + "min " + extSec + "sec");
        }
        Main.debug("[Player.js] - Jump To: [" + extHour + "h " + extMin + "min " + extSec + "sec]");

        if (extHour)
            extHour = parseInt(extHour, 10) * 60 * 60;

        if (extMin)
            extMin = parseInt(extMin, 10) * 60;

        targetTime = extHour + extMin + parseInt(extSec, 10);
        targetTimeInMS = targetTime * 1000;

        if (number && (targetTimeInMS > Player.totalTime || (targetTimeInMS >= Player.wrongTotalTime && Player.wrongTotalTime !== 0))) // red means not possible
            $("#rightOSDJumpToData").css("color", "red");

        return targetTimeInMS;
    }
    return targetTimeInMS;
};

/*
 ###############################
 #
 #       TIME
 #
 ###############################
 */

Player.getMediaDuration = function () {
    var playerDuration = Player.plugin.GetDuration();
    var pmsDuration = (Player.media) ? parseInt((Player.media[Main.selectedMedia]["-duration"] || Player.media[Main.selectedMedia].Part[Player.selectedPart]["-duration"] || 0), 10) : 0;
    var duration = 0;

    if (playerDuration > 0) {
        duration = playerDuration;
    }

    if (pmsDuration === 0) {
        pmsDuration = AutoPlayList.contentDurations[Player.selected];
    }

    //Always use the PMS time , it is more accurate

    if (pmsDuration > 0) {
        duration = pmsDuration;
    }
    else {
        duration = playerDuration;
    }

    Main.debug("[Player.js] - Media Duration: " + duration);


    Player.duration = duration;
    return duration;
};

Player.adjustJumpstep = function (duration) {
    Player.fastforward = parseInt((prefs.player.fastforward || 60), 10);
    Player.rewind = parseInt((prefs.player.rewind || 60), 10);

    var sec = Math.floor(duration / 1000);
    Main.debug("[Player.js] - duration in seconds: " + sec);
    var step = Math.ceil((sec / 20) + 0.1);

    if (step < Player.fastforward) {
        Player.fastforward = step;
        Main.debug("[Player.js] - Decreased Forward jump step, due to length of media: " + step);
    }

    if (step < Player.rewind) {
        Player.rewind = step;
        Main.debug("[Player.js] - Decreased Rewind jump step, due to length of media: " + step);
    }
};

/*
 ###############################
 #
 #       PROGRESS
 #
 ###############################
 */

Player.startProgress = function () {
    Main.debug("[Player.js] - Player starting progress update");
    if (Player.intervalID === 0 && Player.state !== Player.STOPPED) {
        window.clearInterval(Player.intervalID);
        Player.intervalID = window.setInterval(Player.updateProgress, 5000);
    }
};

Player.updateProgress = function () {
    try {
        if (Player.state !== Player.STOPPED) {
            var state = (Player.state === Player.PAUSED) ? "paused" : "playing";
            Main.setProgress(
                AutoPlayList.contentMetadataUrls[Player.selected],
                AutoPlayList.contentRatingKeys[Player.selected],
                Player.curTime,
                Player.duration,
                AutoPlayList.identifier,
                Player.server,
                state,
                function () {
                });
        }
    } catch (e) {

    }
};

/*
 ###############################
 #
 #       3D
 #
 ###############################
 */
Player.checkFileNameFor3D = function () {

    if (!Player.media) {
        Main.debug("[Player.js] - 3D check returned: Unable to check media");
        return 0;
    }

    var mediaFile = "";

    if (Player.mode === Player.MODE_VIDEO) {
        mediaFile = (Player.media[Main.selectedMedia].Part[Player.selectedPart]["-file"] || "");
    }
    else if (Player.mode === Player.MODE_CLIP) {
        mediaFile = (AutoPlayList.contentTitles) ? AutoPlayList.contentTitles[Player.selected] : "";
    }

    if (mediaFile == "") {
        mediaFile = (AutoPlayList.contentCollections) ? AutoPlayList.contentCollections[Player.selected] : "";
    }

    Main.debug("[Player.js] - Player check media for 3D: [" + mediaFile + "]");

    /*
     0 - PL_SCREEN_3DEFFECT_MODE_OFF
     1 - PL_SCREEN_3DEFFECT_MODE_SIDE_BY_SIDE  // doesn't work as samsung says!!
     2 - PL_SCREEN_3DEFFECT_MODE_TOP_BOTTOM
     3 - PL_SCREEN_3DEFFECT_MODE_LINE_BY_LINE
     4 - PL_SCREEN_3DEFFECT_MODE_VERTICAL_STRIPE
     5 - PL_SCREEN_3DEFFECT_MODE_FRAME_SEQUENCE
     6 - PL_SCREEN_3DEFFECT_MODE_CHECKER_BD
     7 - PL_SCREEN_3DEFFECT_MODE_FROM_2D_TO_3D
     */

    mediaFile = mediaFile.toUpperCase(); //makes this case-insensitive

    if (mediaFile.contains("HTAB") || mediaFile.contains("H-TAB") || mediaFile.contains("HALF-TAB") || mediaFile.contains("HALF-OU")) {
        Main.debug("[Player.js] - 3D check detected: TOP-BOTTOM");
        return 2;
    }
    if (mediaFile.contains("HSBS") || mediaFile.contains("H-SBS") || mediaFile.contains("HALF-SBS")) {
        Main.debug("[Player.js] - 3D check detected: SIDE-BY-SIDE");
        return 1;
    }
    if (mediaFile.contains("3D-MVC")) {
        Main.debug("[Player.js] - 3D check detected: MVC");
        return 5;
    }
    if (mediaFile.contains("YT3D")) {
        Main.debug("[Player.js] - 3D check detected: YT3D will use SBS");
        return 1;
    }

    Main.debug("[Player.js] - 3D check returned: Not 3D media");
    return 0;
};

Player.get3D = function () {

    if (Player.mode !== Player.MODE_VIDEO && Player.mode !== Player.MODE_CLIP) {
        return false;
    }

    var pluginScreen = document.getElementById("pluginScreen");
    var pluginTV = document.getElementById("pluginObjectTV");
    var TV = 0;
    var MONITOR = 1;
    var BD = 2;

    if (pluginTV.GetProductType() === TV) // PRODUCT_TYPE_TV
    {
        if (pluginScreen.Flag3DEffectSupport() === 1) // can the TV do 3D
        {
            Main.debug("[Player.js] - TV supports 3D");
            return true;
        }
        else {
            Main.debug("[Player.js] - TV doesn't support 3D", 1);
            Main.showInfo(text.error3DTV, "info");
            return false;
        }
    }
    else if (pluginTV.GetProductType() === BD) // PRODUCT_TYPE_BD
    {
        if (pluginScreen.Flag3DEffectSupport() === 1) // does the device support 3D
        {
            Main.debug("[Player.js] - BD-Player supports 3D");
            return true;
        }
        else {
            Main.debug("[Player.js] - BD-Player doesn't support 3D", 1);
            Main.showInfo(text.error3DBD, "info");
            return false;
        }
    }
    else if (pluginTV.GetProductType() === MONITOR) // PRODUCT_TYPE_MONITOR
    {
        if (pluginScreen.Flag3DEffectSupport() === 1) // can the MONITOR do 3D
        {
            Main.debug("[Player.js] - MONITOR supports 3D");
            return true;
        }
        else {
            Main.debug("[Player.js] - MONITOR doesn't support 3D", 1);
            Main.showInfo(text.error3DMonitor, "info");
            return false;
        }
    }
    else {
        Main.debug("[Player.js] - Unkown Device", 1);
        Main.showInfo(text.error3DUnknown, "info");
        return false;
    }
};

Player.set3D = function (mode) {

    if (Player.mode === Player.MODE_MUSIC) {
        return false;
    }

    var pluginScreen = document.getElementById("pluginScreen");
    var response;
    var result = false;

    Main.debug("[Player.js] - Player will set 3D to: [" + mode + "]");

    if (Player.plugin.sef === true) //2011+ models
    {
        /* On 2011 models
         1  -  PL_SCREEN_3DEFFECT_MODE_SIDE_BY_SIDE
         2  -  PL_SCREEN_3DEFFECT_MODE_TOP_BOTTOM
         */
        switch (mode) {
            case 1:
                response = Player.plugin.Set3DEffectMode(mode); // Note: This is on the player plugin!
                if (response > -1) {
                    Main.debug("[Player.js] - Player 3D mode changed to: SIDE_BY_SIDE");
                    result = true;
                } else {
                    Main.debug("[Player.js] - Player failed to change 3D mode to: SIDE_BY_SIDE");
                    result = false;
                }
                break;
            case 2:
                response = Player.plugin.Set3DEffectMode(mode); // Note: This is on the player plugin!
                if (response > -1) {
                    Main.debug("[Player.js] - Player 3D mode changed to: TOP_BOTTOM");
                    result = true;
                } else {
                    Main.debug("[Player.js] - Player failed to change 3D mode to: TOP_BOTTOM");
                    result = false;
                }
                break;
            default:
                Main.debug("[Player.js] - Player.set3D: mode not set");
                result = false;
        }
    }
    else if (getModel(2010)) //2010 models
    {
        /* On 2010 models
         1  -  PL_SCREEN_3DEFFECT_MODE_TOP_BOTTOM
         2  -  PL_SCREEN_3DEFFECT_MODE_SIDE_BY_SIDE
         */
        switch (mode) {
            case 0:
                response = pluginScreen.Set3DEffectMode(0);
                if (response > -1) {
                    Main.debug("[Player.js] - Player 3D mode changed to: OFF");
                    result = false;
                } else {
                    Main.debug("[Player.js] -  Player failed to change 3D mode to: OFF");
                    result = false;
                }
                break;
            case 1:
                response = pluginScreen.Set3DEffectMode(2);
                if (response > -1) {
                    Main.debug("[Player.js] - Player 3D mode changed to: SIDE_BY_SIDE");
                    result = true;
                } else {
                    Main.debug("[Player.js] -  Player failed to change 3D mode to: SIDE_BY_SIDE");
                    result = false;
                }
                break;
            case 2:
                response = pluginScreen.Set3DEffectMode(1);
                if (response > -1) {
                    Main.debug("[Player.js] - Player 3D mode changed to: TOP_BOTTOM");
                    result = true;
                } else {
                    Main.debug("[Player.js] -  Player failed to change 3D mode to: TOP_BOTTOM");
                    result = false;
                }
                break;
            default:
                result = false;
        }
    }
    else {
        Main.debug("[Player.js] - Player can't change 3D mode, this is an unknown model", 1);
    }
    return result;
};

/*
 ###############################
 #
 #       SET BUFFER
 #
 ###############################
 */

Player.setBuffer = function () {
    var initBuffer = parseInt((prefs.player.initBuffer || 0), 10);
    var pendingBuffer = parseInt((prefs.player.pendingBuffer || 0), 10);
    var totalBuffer = parseInt((prefs.player.totalBuffer || 0), 10);
    var result;

    if (initBuffer > 0) {
        result = Player.plugin.SetInitialBuffer(initBuffer * 1024 * 1024);
        if (result) {
            Main.debug("[Player.js] - Player Init Buffer: [" + initBuffer + " mb]");
        } else {
            Main.debug("[Player.js] - Player set Init Buffer failed!");
        }
    } else {
        Main.debug("[Player.js] - Player Init Buffer: [auto]");
    }

    if (pendingBuffer > 0) {
        result = Player.plugin.SetPendingBuffer(pendingBuffer * 1024 * 1024);
        if (result) {
            Main.debug("[Player.js] - Player Pending Buffer: [" + pendingBuffer + " mb]");
        } else {
            Main.debug("[Player.js] - Player set Pending Buffer failed!");
        }
    } else {
        Main.debug("[Player.js] - Player Pending Buffer: [auto]");
    }

    if (totalBuffer > 0) {
        result = Player.plugin.SetTotalBufferSize(totalBuffer * 1024 * 1024);
        if (result) {
            Main.debug("[Player.js] - Player Total Buffer: [" + totalBuffer + " mb]");
        } else {
            Main.debug("[Player.js] - Player set Total Buffer failed!");
        }
    } else {
        Main.debug("[Player.js] - Player Total Buffer: [auto]");
    }
};

Player.setAspectButtonText = function () {
    if (Player.aspectMode === Player.ASPECT_DEFAULT)
        widgetAPI.putInnerHTML(document.getElementById("rightOSDAspectRatio"), text.OSDAspectDefault);
    else if (Player.aspectMode === Player.ASPECT_STRETCH)
        widgetAPI.putInnerHTML(document.getElementById("rightOSDAspectRatio"), text.OSDAspectStretch);
    else if (Player.aspectMode === Player.ASPECT_ZOOM)
        widgetAPI.putInnerHTML(document.getElementById("rightOSDAspectRatio"), text.OSDAspectZoom);
};

/*
 ###############################
 #
 #       FRONT PANEL
 #
 ###############################
 */

Player.frontPanelSetTime = function (timeHour, timeMinute, timeSecond) {
    var fpPlugin = document.getElementById("pluginFrontPanel");
    try {
        fpPlugin.DisplayVFD_Time(timeHour, timeMinute, timeSecond);
    }
    catch (err) {
    }

};

Player.frontPanelSetStatus = function (status) {
    var fpPlugin = document.getElementById("pluginFrontPanel");
    try {
        fpPlugin.DisplayVFD_Show(status);
    }
    catch (err) {
    }
};

/*
 ###############################
 #
 #       BUFFERING
 #
 ###############################
 */

Player.onBufferingStart = function () {
    Main.debug("[Player.js] - Player Buffering...");
    window.clearInterval(Player.intervalID);//stop updating
    Player.intervalID = 0; //reset so it will resume after buffering

    keyLock.status = keyLock.PLAYERSTART;
    Player.state = Player.BUFFER;
    if (Player.mode === Player.MODE_MUSIC) {
        loadbar.show();
    }
    else {
        loading.show();
    }
};

Player.onBufferingProgress = function (percent) {
    Main.debug("[Player.js] - Player Buffering: [" + percent + "%]");
    setTimeout(function () {
        if (percent > 50) {
            loadbar.hide();
            loading.hide();
            if (Player.mode !== Player.MODE_MUSIC) {
                $('#background').hide();
            }
        }
    }, 0);
};

Player.onBufferingComplete = function () {
    Main.debug("[Player.js] - Player Buffering Complete");
    loadbar.hide();
    loading.hide();
    if (Player.mode !== Player.MODE_MUSIC) {
        $('#background').hide();
    }
};

/*
 ###############################
 #
 #       PLAYBACK EVENTS
 #
 ###############################
 */

Player.delayedJump = function () {
    if (Player.jumpVar === 1) {
        return false;
    }

    if (Player.mode === Player.MODE_VIDEO) {  //doesn't work well with clips
        if (Player.jumpVar === 0) {// single check is faster
            Main.debug("[Player.js] - delayedJump Player.curTime: [" + Player.curTime / 1000 + "] Player.resumeTime [" + Player.resumeTime + "]");
            Player.adjustJumpstep(Player.getMediaDuration());
            if ((Player.curTime > 5000) || (Player.resumeTime === 0)) {// if we got this far, no jump is needed or it already happened
                Player.jumpVar = 1;
                Main.debug("[Player.js] - Delayed Jump canceled by timeout");
                keyLock.status = keyLock.NONE;
                return false;
            }

            if ((Player.curTime > 0) && (Player.curTime / 1000 < (Player.resumeTime))) {
                if (Player.resumeTime > Player.totalTime) {
                    Main.debug("[Player.js] - Jump canceled, player wanted to jump from: [" + Player.curTime / 1000 + "] to [" + Player.resumeTime + "] , but media duration = [" + Player.totalTime + "]");
                    keyLock.status = keyLock.NONE;
                } else {
                    Main.debug("[Player.js] - Player will jump from: [" + Player.curTime / 1000 + "] to [" + Player.resumeTime + "]");
                    Player.plugin.JumpForward(Player.resumeTime);
                    keyLock.status = keyLock.NONE;
                }
            } else if (Player.curTime === 0) {
                Main.debug("[Player.js] - Player time could not be determined, Jump canceled");
                keyLock.status = keyLock.NONE;
            } else {
                Main.debug("[Player.js] - Player time: [" + Player.curTime / 1000 + "] already beyond [" + Player.resumeTime + "], Jump canceled");
                keyLock.status = keyLock.NONE;
            }
            Player.jumpVar = 1;
        }
    }
    else if (Player.mode === Player.MODE_CLIP) {
        if (Player.jumpVar === 0) { // redo a few things, because the actual numbers get in late with an online stream
            Player.adjustJumpstep(Player.getMediaDuration());
            Player.setWindow();
            Player.jumpVar = 1;
            Main.debug("[Player.js] - Display reset by Delayed Jump");
            keyLock.status = keyLock.NONE;
        }
    }
    else {
        Main.debug("[Player.js] - Delayed Jump canceled: Player not in correct mode");
        Player.jumpVar = 1;
        keyLock.status = keyLock.NONE;
    }
    return false;
};

Player.TimeHandler = function (currentTime) {
    try {
        Display.setTime(currentTime);
        Player.curTime = currentTime;

        if (Player.state == Player.BUFFER) {  //player is buffering, but video is already playing, hide the loading gif
            Main.debug("[Player.js] - Player Content is playing");
            window.clearTimeout(Player.jumpTimer);
            Player.jumpTimer = window.setTimeout(Player.delayedJump, 1000);
            Player.startProgress();
            loading.hide();
            Player.state = Player.PLAYING;
            keyLock.status = keyLock.NONE;
        }

        if (Player.state === Player.REWIND && time === 0) { // when you rewind and you hit the start the content starts playing auto. so we have to change the status pic
            Player.state = Player.PAUSED;
            Player.pauseVideo();
        }

    } catch (e) {
        Main.debug("[Player.js] - Player.TimeHandler error: " + e.message());
    }

};

Player.subtitleHandler = function (time) {
    try {
        if (Player.state !== Player.PAUSED) { // because on 2010 device the device triggers this function even if video is paused
            if (Player.transcoding === 0) {
                Subtitle.onSubTitle(time);
            }
            else if (Player.transcoding === 1 && Main.getPlaybackMode() !== 4) {
                Subtitle.onSubTitle(time);
            }
        }
    } catch (e) {
        Main.debug("[Player.js] - Player.subtitleHandler error: " + e.message());
    }
};

/*
 ###############################
 #
 #       EVENTS
 #
 ###############################
 */

Player.onStreamInfoReadyHandler = function () {
    Player.totalTime = Player.getMediaDuration();
    Main.debug("[Player.js] - Player Set Total Time to: [" + Player.totalTime + "ms] - [" + (Player.totalTime / 1000) + "sec] - [" + Math.round(Player.totalTime / 1000 / 60) + "min]");

    if (Player.plugin.sef !== true) { //only for 2010 models
        if (Player.screenMode > 0 && Player.stereoScopic === false) {
            Player.stereoScopic = Player.set3D(Player.screenMode);
        }
    }
};

Player.onCurrentPlayTimeHandler = function (time) {//this is triggered with a 500 ms rate, but time only changes per second
    try {
        var currentTime = Number(time);
        if (currentTime > 0) {
            if (Player.mode === Player.MODE_VIDEO && Player.state === Player.PLAYING) {
                Player.subtitleHandler(currentTime); //always update subtitles, it has a half second trick to enhance accuracy
            }

            if (currentTime !== Player.curTime) {
                Player.TimeHandler(currentTime); // time update is useless unless the time changed, save on cycles
            }
        }
    } catch (e) {
        Main.debug("[Player.js] - Player.onCurrentPlayTimeHandler error: " + e.message());
    }
};

Player.onRenderingComplete = function () //This does not work like it should, don't use it
{
    window.clearInterval(Player.intervalID);
    Player.stopVideo(Main.WATCHED);
};

Player.onNetworkDisconnected = function () // works
{
    Main.debug("[Player.js] - Player Network Error!", 1);
    Player.stopVideo(Main.NETWORKERROR);
    Main.showInfo(text.errorNetwork);
};

Player.onRenderStart = function () {
    Main.debug("[Player.js] - Player Rendering Start");
};

Player.onRenderError = function (number) {
    Player.errorCount++;
    Main.debug("[Player.js] - Player Render Error! [" + number + "]", 1);
    Player.checkTimeoutID = window.setTimeout(Player.getErrorMessage, 2000, number);
};

Player.getErrorMessage = function (number) {
    var errorMessage;
    switch (Number(number)) {
        case 1:
            Main.debug("[Player.js] - Player Render Error: [" + text.errorContainer + "]", 1);
            errorMessage = text.errorContainer + "<br>" + text.errorUnableToPlay;
            break;
        case 2:
            Main.debug("[Player.js] - Player Render Error: [" + text.errorVideoCodec + "]", 1);
            errorMessage = text.errorVideoCodec + "<br>" + text.errorUnableToPlay;
            break;
        case 3:
            Main.debug("[Player.js] - Player Render Error: [" + text.errorAudioCodec + "]", 1);
            errorMessage = text.errorAudioCodec + "<br>" + text.errorUnableToPlay;
            break;
        case 4:
            Main.debug("[Player.js] - Player Render Error: [" + text.errorVideoRes + "]", 1);
            errorMessage = text.errorVideoRes + "<br>" + text.errorUnableToPlay;
            break;
        case 5:
            Main.debug("[Player.js] - Player Render Error: [" + text.errorVideoFramerate + "]", 1);
            errorMessage = text.errorVideoFramerate + "<br>" + text.errorUnableToPlay;
            break;
        case 6:
            Main.debug("[Player.js] - Player Render Error: [" + text.errorVideoCorrupted + "]", 1);
            errorMessage = text.errorVideoCorrupted;
            break;
        default:
            Main.debug("[Player.js] - Player Unknown Render error " + Player.errorCount, 1);
            errorMessage = text.errorUnknown + "<br>" + text.errorUnableToPlay;
            break;
    }

    // greater than 0 when we started movie from beginning or greater than resumetime if we started movie from resume point
    if ((Player.curTime > 0 || Player.curTime > Player.resumeTime) && Player.errorCount > 4) {
        Main.debug("[Player.js] - Player Render Error but content is playing so showing whats the problem is", 1);
        if (Player.state !== Player.PAUSED) {
            Player.pauseVideo();
        }
        Player.errorCount = 0;
        Main.showInfo(errorMessage);
    } else if (Player.errorCount > 1) {
        Main.debug("[Player.js] - Player Render Error is true! Will stop video", 1);
        Player.stopVideo(Player.ERROR);
        Player.errorCount = 0;
        Main.showInfo(errorMessage, "error");
    }
};

Player.onConnectionFailed = function () //works if there is no file anymore
{
    Main.debug("[Player.js] - Player Connection Failed!", 1);
    if (Player.themePlaying === 1) {
        Player.stopMusic();
    } else {
        Player.stopVideo(Player.ERROR);
        Main.showInfo(text.errorConnection);
    }
};

Player.onStreamNotFound = function () {
    Main.debug("[Player.js] - Player Stream Not Found!", 1);
    if (Player.themePlaying === 1) {
        Player.stopMusic();
    } else {
        Player.stopVideo(Player.ERROR);
        Main.showInfo(text.errorStreamNotFound);
    }

};

Player.onAuthenticationFailedHandler = function () {
    Main.debug("[Player.js] - Player Authentication Failed!", 1);
    Player.stopVideo(Player.ERROR);
    Main.showInfo(text.errorPMS401);
};