function Server(machID) {
    var x = {
        accessToken: "",
        name: "",
        address: "",
        port: "",
        version: "",
        scheme: "",
        host: "",
        localAddresses: [],
        machineIdentifier: "",
        createdAt: "",
        updatedAt: "",
        owned: false,
        synced: false,
        url: "",
        ip: "",
        reachable: false,
        allowMediaDeletion: false,
        flashInstalled: 0,
        friendlyName: "",
        multiuser: false,
        platform: "",
        platformVersion: "",
        silverlightInstalled: false,
        transcoderActiveVideoSessions: 0,
        transcoderAudio: false,
        transcoderVideo: false,
        transcoderVideoBitrates: "",
        transcoderVideoQualities: "",
        transcoderVideoResolutions: "",
        webkit: false,
        musicCount: 0,
        photosCount: 0,
        videoCount: 0,
        shared: false,

        features: {
            universalTranscoder: '0.9.8.9',
            universalMusicTranscoder: '0.9.8.18'
        },

        init: function (id) {
            if (id == "https://plex.tv") {
                this.accessToken = (prefs.myplex.accessToken || "");
                this.name = "Plex.TV";
                this.address = "plex.tv";
                this.port = "443";
                this.version = "0.9.8.0";
                this.scheme = "https";
                this.host = "plex.tv";
                this.localAddresses = [];
                this.machineIdentifier = id;
                this.createdAt = "";
                this.updatedAt = "";
                this.owned = false;
                this.synced = false;
                this.url = "https://plex.tv";
                this.ip = "plex.tv";
                this.reachable = true;
                this.allowMediaDeletion = true;
                this.flashInstalled = false;
                this.friendlyName = "Plex Account";
                this.multiuser = false;
                this.platform = "";
                this.platformVersion = "";
                this.silverlightInstalled = false;
                this.transcoderActiveVideoSessions = 0;
                this.transcoderAudio = false;
                this.transcoderVideo = false;
                this.transcoderVideoBitrates = "";
                this.transcoderVideoQualities = "";
                this.transcoderVideoResolutions = "";
                this.webkit = false;
                this.musicCount = 0;
                this.photosCount = 0;
                this.videoCount = 0;
            } else {
                for (var i = 0; i < prefs.servers.length; i++) {
                    if (prefs.servers[i].machineIdentifier == id) {
                        this.accessToken = (prefs.servers[i].accessToken || "");
                        this.name = (prefs.servers[i].name || "");
                        this.address = (prefs.servers[i].address || "");
                        this.port = (prefs.servers[i].port || "32400");
                        this.version = (prefs.servers[i].version || "0.9.8.0");
                        this.scheme = (prefs.servers[i].scheme || "http");
                        this.host = (prefs.servers[i].host || "");
                        this.localAddresses = (prefs.servers[i].localAddresses || []);
                        this.machineIdentifier = (prefs.servers[i].machineIdentifier || "");
                        this.createdAt = (prefs.servers[i].createdAt || "");
                        this.updatedAt = (prefs.servers[i].updatedAt || "");
                        this.owned = (prefs.servers[i].owned == 1);
                        this.synced = (prefs.servers[i].synced == 1);
                        this.url = (prefs.servers[i].url || "");
                        this.ip = (prefs.servers[i].ip || "");
                        this.reachable = (prefs.servers[i].synced === "yes");
                        this.allowMediaDeletion = (prefs.servers[i].allowMediaDeletion == 1);
                        this.flashInstalled = (prefs.servers[i].flashInstalled == 1);
                        this.friendlyName = (prefs.servers[i].friendlyName || "");
                        this.multiuser = (prefs.servers[i].multiuser == 1);
                        this.platform = (prefs.servers[i].platform || "");
                        this.platformVersion = (prefs.servers[i].platformVersion || "");
                        this.silverlightInstalled = (prefs.servers[i].silverlightInstalled == 1);
                        this.transcoderActiveVideoSessions = Number(prefs.servers[i].transcoderActiveVideoSessions || 0);
                        this.transcoderAudio = (prefs.servers[i].transcoderAudio == 1);
                        this.transcoderVideo = (prefs.servers[i].transcoderVideo == 1);
                        this.transcoderVideoBitrates = (prefs.servers[i].transcoderVideoBitrates || "");
                        this.transcoderVideoQualities = (prefs.servers[i].transcoderVideoQualities || "");
                        this.transcoderVideoResolutions = (prefs.servers[i].transcoderVideoResolutions || "");
                        this.webkit = (prefs.servers[i].webkit == 1);
                        this.musicCount = (prefs.servers[i].musicCount || 0);
                        this.photosCount = (prefs.servers[i].photosCount || 0);
                        this.videoCount = (prefs.servers[i].videoCount || 0);
                        break;
                    }
                }
            }
        },

        supports: function (feature) {
            var minVersion = this.features[feature];
            var serverVersion = this.version;

            if (minVersion && serverVersion) {
                minVersion = minVersion.split('.');
                serverVersion = serverVersion.split('.');

                for (var i = 0, len = minVersion.length; i < len; i++) {
                    if (i < serverVersion.length) {
                        var minVersionDot = parseInt(minVersion[i], 10);
                        var serverVersionDot = parseInt(serverVersion[i], 10);

                        if (minVersionDot < serverVersionDot)
                            return true;
                        else if (minVersionDot > serverVersionDot)
                            return false;
                    }
                }
                return true;
            } else {
                return true;
            }
        },

        getHeaders: function () {
            var type = getProductType();
            var model = type + " " + Main.getModelName();
            var UID = Main.getUniqueID();

            var headers = {
                'X-Plex-Version': Application.appVersion,
                'X-Plex-Platform': 'Samsung',
                'X-Plex-Client-Platform': 'Samsung',
                'X-Plex-Device': 'Samsung ' + type,
                'X-Plex-Device-Name': model,
                'X-Plex-Client-Identifier': encodeURIComponent(UID)
            };

            if ((prefs.myplex.username || "") !== "") {
                headers["X-Plex-Username"] = prefs.myplex.username;
            }

            return headers;
        },

        replaceBaseUrl: function (url, base) {
            var ipp = new RegExp("/[0-9]{3}.[0-9]{3}.[0-9]{3}.[0-9]{3}/");
            if (url.match(ipp)) {
                url.markup = url.markup.replace(ipp, base);
            }
            return url;
        },

        getRemoteAddress: function () {
            return  this.scheme + "://" + this.address + ":" + this.port;
        },

        getLocalAddress: function () {
            if (this.localAddresses.length > 0) {
                return this.localAddresses[0];
            } else {
                return "";
            }
        },

        getLocalServerCount: function () {
            return this.getLocalServers().length;
        },

        getLocalServers: function () {
            return _.filter(prefs.servers, function (srv) {
                return ((Number((srv["owned"] || "0")) === 1) && (Number((srv["synced"] || "0")) === 0));
            });
        },

        getLocalMachIDs: function () {
            var localServers = _.filter(prefs.servers, function (srv) {
                return (Number((srv["owned"] || "0")) === 1);
            });
            return _.pluck(localServers, 'machineIdentifier');
        },

        getTranscodeServer: function (type, local) {

            //if the current server can handle it, return that one
            if (type === "video") {
                if (this.transcoderVideo && this.reachable) {
                    return this.machineIdentifier;
                }
            } else if (type === "audio") {
                if (this.transcoderVideo && this.reachable) {
                    return this.machineIdentifier;
                }
            }

            //see if we have an alternative one that can do the transcode
            //if local is true then we only want a local server, not the remote ones
            //but always prefer a local over a remote one
            var i;
            for (i = 0; i < prefs.servers.length; i++) {
                if (this.isOwned(i) && !this.isSynced(i) && isPrivate(prefs.servers[i].ip) && this.isReachable(i)) {
                    if (type === "video") {
                        if (prefs.servers[i].transcoderVideo) {
                            return prefs.servers[i].machineIdentifier;
                        }
                    } else if (type === "audio") {
                        if (prefs.servers[i].transcoderAudio) {
                            return prefs.servers[i].machineIdentifier;
                        }
                    } else {
                        //they can all manage images
                        return prefs.servers[i].machineIdentifier;
                    }
                }
            }

            //try the remote servers
            if (!local) {
                for (i = 0; i < prefs.servers.length; i++) {
                    if (!this.isOwned(i)) { //remote server
                        if (type === "video") {
                            if (prefs.servers[i].transcoderVideo) {
                                return prefs.servers[i].machineIdentifier;
                            }
                        } else if (type === "audio") {
                            if (prefs.servers[i].transcoderAudio) {
                                return prefs.servers[i].machineIdentifier;
                            }
                        } else {
                            //they can all manage images
                            return prefs.servers[i].machineIdentifier;
                        }
                    }
                }
            }

            //no alternative server avaialable for transcoding
            return null;
        },

        transcodeImage: function () {
            if (this.transcoderVideo) {
                return true
            }

            if (isPrivate(this.ip)) {
                return true;
            }

            return false;
        },

        IsUsingLocalIP: function () {
            return isPrivate(this.ip);
        },

        isReachable: function (idx) {
            if (prefs.servers[idx])
                return ((prefs.servers[idx].reachable || "no") === "yes") ? true : false;
            else
                return false;
        },

        isOwned: function (idx) {
            if (prefs.servers[idx])
                return ((prefs.servers[idx].owned || "0") === "1") ? true : false;
            else
                return false;
        },

        isSynced: function (idx) {
            if (prefs.servers[idx])
                return ((prefs.servers[idx].synced || "0") === "1") ? true : false;
            else
                return false;
        },

        hasMoreLocalAddresses: function (addressIdx) {
            return (this.localAddresses.length < (addressIdx + 1))
        },

        getIndexByIdentifier: function (machineIdentifier) {
            for (var i = 0; i < prefs.servers.length; i++) {
                if (prefs.servers[i].machineIdentifier === machineIdentifier) {
                    return i;
                }
            }
            return -1;
        },

        getIndexByAddress: function (address) {
            for (var i = 0; i < prefs.servers.length; i++) {
                if (prefs.servers[i].address === address) {
                    return i;
                }
            }
            return -1;
        },

        getNameByIdentifier: function (machineIdentifier) {
            for (var i = 0; i < prefs.servers.length; i++) {
                if (prefs.servers[i].machineIdentifier === machineIdentifier) {
                    return prefs.servers[i].name;
                }
            }
            return "Unknown";
        },

        getUrl: function (addresIdx) {
            var url = "";
            var ip = "";

            if (this.url != "" && isNaN(addresIdx)) {
                return this.url;
            } else if (this.owned && this.synced) { //cloud sync
                url = this.scheme + "://" + this.host + ":" + this.port;
            } else if (this.owned && !this.synced) { //local
                if (!isNaN(addresIdx)) {
                    ip = this.localAddresses[addresIdx];
                    if (ip === "") {
                        url = this.getRemoteAddress();
                    } else {
                        url = "http://" + ip + ":32400";
                    }
                } else {
                    url = this.getRemoteAddress();
                }
            } else { //remote
                url = this.getRemoteAddress();
            }
            return url;
        },

        resolveUrl: function (path, useLoopback, appendToken) {

            //Main.debug("[Server.js] - path: " + path);
            var pms = this.url;
            var loopback = "http://127.0.0.1:32400";
            var fullUrl = "";
            var base = (Data.category[Main.selectedCat].baseUrl || "");
            var part = ('/:/|/library/|/channels/|/music/|/video/|/photos/|/system/|/sync/');
            base = (path.match(part) === null) ? base : "";

//            Main.debug("[Server.js] - pms: " + pms);
//            Main.debug("[Server.js] - name: " + this.name);
//            Main.debug("[Server.js] - Base URL: " + base);
//            Main.debug("[Server.js] - resolveUrl: " + path);

            if (path.contains("plex://127.0.0.1")) {
                Main.debug("[Main.js] - webkit Url found, replacing url");
                fullUrl = path.replace("plex://127.0.0.1", pms);
            } else if (/^\w+:\/\//.test(path)) {
                fullUrl = path;
            } else if (useLoopback) {
                fullUrl = loopback.join(path);
            } else if (base !== "" && !path.startsWith(base)) {
                fullUrl = pms.join(base).join(path);
            } else {
                fullUrl = pms.join(path);
            }

            if (appendToken) {
                fullUrl = this.appendToken(fullUrl);
            }

            //Main.debug("[Server.js] - resolved Url: " + fullUrl);
            return fullUrl;
        },

        appendToken: function (path) {

            if (isEmpty(path)) {
                return false;
            }

            var token;
            var fullUrl = path;
            var chk = ('\\?X-Plex-Token|\\&X-Plex-Token');

            if (path.match('plex.tv') !== null) {
                token = (prefs.myplex.authToken || "");
            } else {
                token = this.accessToken;
            }

            if (isEmpty(token)) {
                return fullUrl;
            }

            //Main.debug("Server.js - current token: " + token);

            if (path.match(chk) !== null) {
                //Main.debug("Server.js - token already exists: " + qToken);
                return fullUrl;
            }

            if (!isEmpty(token)) {
                fullUrl += (fullUrl.indexOf('?') > 0) ? '&' : '?';
                fullUrl += 'X-Plex-Token=' + token;
            }
            return fullUrl;
        }
    };

    x.init(machID);
    return x;
}