var Subtitle =
{
    subtitleIndex: 0,
    subtitleUrl: "",
    subtitles: [],
    currentSubtitle: -1,
    subSyncOffset: 0,
    currentSubTime: 0,
    enabled: false,
    loading: false
};

Subtitle.init = function (callback) {

    //Audio
    Audio.audioIndex = 0; //reset
    var audioCount = Metadata.getAudioCount();
    Main.debug("[Subtitle.js] - AudioCount: [" + audioCount + "]");

    Audio.audioIndex = Metadata.getSelectedAudio();
    Main.debug("[Subtitle.js] - Audio.audioIndex: [" + Audio.audioIndex + "]");

    if (Audio.audioIndex > -1) {// if its not the first track then we have to change it
        if (Player.plugin.sef && Player.transcoding === 0) {
            var res = Player.plugin.SetStreamID(1, Audio.audioIndex); // change the Audio
            Main.debug("[Subtitle.js] - Set Audio Stream to: " + Audio.audioIndex + ", result: " + (res < 0 ? "Failed" : "Success"));
            if (res > 0) // if its 1 it worked
            {
                Audio.setAudioOutput(AudioStreams[Audio.audioIndex].codec); // change the output to receiver
            }
        }
        else if (Player.transcoding === 1) {
            Audio.setAudioOutput(AudioStreams[Audio.audioIndex].codec); // change the output to receiver
        }
    }

    // init the rightOSD
    if (Metadata.getAudioCount() > 0)
        insertText("rightOSDAudioStreamData", (Audio.audioIndex + 1) + "/" + Metadata.getAudioCount() + " " + Metadata.getAudioLanguage(Audio.audioIndex) + " - " + Metadata.getAudioCodec(Audio.audioIndex) + " " + Metadata.getAudioChannels(Audio.audioIndex));
    else
        insertText("rightOSDAudioStreamData", "none");

    //Subtitles
    Main.debug("[Subtitle.js] - SubtitleCount: [" + Metadata.getSubtitleCount() + "]");

    if (Metadata.getSubtitleCount() === 0) {
        insertText("rightOSDSubtitleStreamData", "none");
    }

    if (Metadata.getSubtitleCount() > 0) {
        Subtitle.subtitleIndex = Metadata.getSelectedSubtitle(); //load the subtitle selected by the PMS or else the first available

        if (this.subtitleIndex !== -1) { // we have subs and a selected one
            Main.debug("[Subtitle.js] - Preferred subtitle loaded: " + this.subtitleIndex);
            Subtitle.setSubtitle(Subtitle.subtitleIndex, true, function () {
                if (callback) {
                    callback();
                }
            });
            return false;
        } else { // we have subs but not a preffered one, so load the first available
            Subtitle.subtitleIndex = 0;
            Main.debug("[Subtitle.js] - First available subtitle loaded");
            Subtitle.setSubtitle(Subtitle.subtitleIndex, true, function () {
                if (callback) {
                    callback();
                }
            });
            return false;
        }
    }
    else {
        Subtitle.resetOnScreenSubtitle();
        Subtitle.subtitleIndex = -1;
        Subtitle.setSubtitle(Subtitle.subtitleIndex, false, function () {
            if (callback) {
                callback();
            }
        });
        return false;
    }
};

Subtitle.isSupported = function (index) {
    if (SubtitleStreams[index]) {
        if ((Main.getPlaybackMode() > 2) && Main.newTranscoderEnabled()) {
            return true;
        } else if ((Main.getPlaybackMode() > 2) && (SubtitleStreams[index].format === "srt" || SubtitleStreams[index].format === "smi")) {
            return true;
        } else if ((Main.getPlaybackMode() <= 2) && (SubtitleStreams[index].format === "srt" || SubtitleStreams[index].format === "smi") && SubtitleStreams[index].embedded === 0) {
            return true;
        } else {
            return false;
        }
    } else {
        return false;
    }
};

Subtitle.canBeDisplayed = function (index) {
    if (SubtitleStreams[index]) {
        if ((SubtitleStreams[index].format === "srt" || SubtitleStreams[index].format === "smi") && SubtitleStreams[index].embedded === 0) {
            return true;
        } else {
            return false;
        }
    } else {
        return false;
    }
};

Subtitle.toggleEnabled = function () {
    if (Subtitle.enabled === true) {
        Subtitle.enabled = false;
    } else {
        Subtitle.enabled = Subtitle.checkEnable(Subtitle.subtitleIndex, true);
    }
    Subtitle.refreshRightOSD(); // set enabled-disabled items
};

Subtitle.checkEnable = function (index, manual) {
    var isTranscode = (Main.getPlaybackMode() > 2) ? true : false;
    var canDisplay = Subtitle.canBeDisplayed(index);
    var prefOverlay = ((prefs.subtitles.preferOverlay || "off") === "on") ? true : false;
    var alwaysOn = ((prefs.subtitles.alwaysEnabled || "off") === "on") ? true : false;

    if (!canDisplay) {
        Main.debug("[Subtitle.js] - Can not enable overlayed subtitle at this time");
        return false;
    } else if (isTranscode) {
        if (!prefOverlay) {
            Main.debug("[Subtitle.js] - Using embedded subtitle because this is the preferred setting");
            return false;
        } else if (manual) {
            return true;
        } else {
            if (alwaysOn) {
                Main.debug("[Subtitle.js] - Using overlayed subtitle because this is the preferred setting");
            }
            return alwaysOn;
        }
    } else {
        if (manual) {
            return true;
        } else {
            return alwaysOn;
        }
    }
};

Subtitle.refreshRightOSD = function () {
    var i;

    if (Subtitle.canBeDisplayed(Subtitle.subtitleIndex)) {
        $("#rightOSDTable").find("tr:nth-child(" + 3 + ")").children().css("color", "grey");
    } else {
        $("#rightOSDTable").find("tr:nth-child(" + 3 + ")").children().css("color", "#454545");
    }

    if (Subtitle.canBeDisplayed(Subtitle.subtitleIndex)) {
        for (i = 3; i <= 7; i++) { // if we have no subs then we grey out the sub setting so the users know that he cant use them
            $("#rightOSDTable").find("tr:nth-child(" + (i + 1) + ")").children().css("color", "grey");
        }
    } else {
        for (i = 3; i <= 7; i++) {
            $("#rightOSDTable").find("tr:nth-child(" + (i + 1) + ")").children().css("color", "#454545");
        }
    }

    if (Subtitle.enabled === true) {
        widgetAPI.putInnerHTML(document.getElementById("rightOSDSubtitleEnableData"), "<img src='Images/setting/on.png'>");
    } else {
        widgetAPI.putInnerHTML(document.getElementById("rightOSDSubtitleEnableData"), "<img src='Images/setting/off.png'>");
    }

    //set first item highlighted
    $("#rightOSDTable").find("tr:nth-child(" + (Main.selectedRightOSD + 1) + ")").children().css("color", "white");
};


Subtitle.setSubtitle = function (index, load, callback) {
    Subtitle.resetOnScreenSubtitle();
    Main.debug("[Subtitle.js] - Subtitle index: [" + index + "]");

    if (Metadata.getSubtitleCount() > 0) {
        var embedded = (SubtitleStreams[index].embedded === 1) ? text.internal : text.external;
        $("#rightOSDSubtitleStreamData").putInnerHTML((Subtitle.subtitleIndex + 1) + "/" + Metadata.getSubtitleCount() + " " + SubtitleStreams[index].language + "&nbsp;(" + SubtitleStreams[index].format.toUpperCase() + "&nbsp;" + embedded + ")");
    } else {
        $("#rightOSDSubtitleStreamData").putInnerHTML("0/0" + "&nbsp;" + text.none);
    }

    if (index < 0) // < 0 = NO SUBTITLES
    {
        Subtitle.enabled = false;
        Main.debug("[Subtitle.js] - Subtitles have been disabled");
    }
    else {
        var subUrl = Metadata.getSubtitleUrl(index);
        if (subUrl !== "") {
            if (Subtitle.canBeDisplayed(index)) {

                Subtitle.enabled = Subtitle.checkEnable(Subtitle.subtitleIndex);

                if (Main.rightOSD === 0) {
                    insertText("playerStatus", text.playerSubIs + (Subtitle.subtitleIndex + 1) + "/" + Metadata.getSubtitleCount() + " " + SubtitleStreams[index].language + " [" + SubtitleStreams[index].format.toUpperCase() + "]");
                }

                if (load) {
                    Subtitle.refreshRightOSD();
					
					if(Player.server === null){
						Player.server = Main.server;
					}

                    if (Subtitle.subtitleUrl.contains("?"))
                        Subtitle.subtitleUrl = Player.server.resolveUrl(subUrl + "&encoding=utf-8&format=srt", false, true);
                    else
                        Subtitle.subtitleUrl = Player.server.resolveUrl(subUrl + "?encoding=utf-8&format=srt", false, true);

                    Subtitle.subtitles = [];
                    Subtitle.loadSubtitles(callback); //the callback of this will call Subtitle.parseSrt;
                    return false;
                }
                Main.debug("[Subtitle.js] - Loaded Subtitle: [" + (Subtitle.subtitleIndex + 1) + "] of: [" + Metadata.getSubtitleCount() + "]");
            } else {
                Subtitle.enabled = false;
                Main.debug("[Subtitle.js] - Subtitle set to: [" + (Subtitle.subtitleIndex + 1) + "] of: [" + Metadata.getSubtitleCount() + "]");
            }

            // $("#rightOSDSubtitleStreamData").putInnerHTML((Subtitle.subtitleIndex+1)+ "/" + Metadata.getSubtitleCount() + " " + SubtitleStreams[index].language + "&nbsp;(" + SubtitleStreams[MediaInfo.subtitle].format.toUpperCase() + "&nbsp;" + embedded + ")");

            if (Main.rightOSD === 0) {
                insertText("playerStatus", text.playerSubIs + (Subtitle.subtitleIndex + 1) + "/" + Metadata.getSubtitleCount() + " " + SubtitleStreams[index].language + " [" + SubtitleStreams[index].format.toUpperCase() + "]");
            }
        }
        else {
            Subtitle.enabled = false;
            Subtitle.resetOnScreenSubtitle();
            Main.debug("[Subtitle.js] - No Subtitle set for overlay");
        }
    }

    Subtitle.refreshRightOSD();

    if (callback) {
        callback();
    }
};

Subtitle.resetOnScreenSubtitle = function () {
    $("#srtSubtitle, #srtSubtitle i, #srtSubtitle b").html('');
    Subtitle.subtitles = [];
    Subtitle.currentSubTitle = -1;
    Subtitle.subtitleUrl = "";
};

Subtitle.loadSubtitles = function (callback) {
    try {
        if (Subtitle.subtitleUrl !== "") {
            Subtitle.loading = true;
            Main.debug("[Subtitle.js] - Loading subtitles from: [" + Subtitle.subtitleUrl + "]");
            AjaxManager.getHTTP(Subtitle.subtitleUrl, Subtitle.parseSrt, null, 10000, "get_subtitles");
        }
        else {
            Subtitle.loading = false;
            Main.debug("[Subtitle.js] - No subtitles loaded, starting playback", 1);
        }
        if (callback) {
            callback();
        }
    }
    catch (err) {
        Subtitle.loading = false;
        Main.debug("[Subtitle.js] - Load Subtitles failed, starting playback", 1);
        if (callback) {
            callback();
        }
    }
};

Subtitle.parseSrt = function (tree, xhr, id) {
    var srt;
    var srtdata;
    if (xhr) {
        if (xhr.status === 200) {
            srtdata = xhr.responseText;
            Main.debug("[Subtitle.js] - Parsing Srt Subtitles started");
            srt = srtdata.replace(/\r\n|\r|\n/g, '\n');
            srt = Subtitle.strip(srt);
            var srtline = srt.split('\n\n');
            if (srtline.length > 0) {
                for (var s = 0; s < srtline.length; s++) {
                    var st = srtline[s].split('\n');
                    //Main.debug("[Subtitle.js] - parseLine -> " + st);
                    Subtitle.parseLine(st);
                }
            }
            Subtitle.loading = false;
            Main.debug("[Subtitle.js] - Parsing Srt Subtitles finished");
        } else {
            Main.debug("[Subtitle.js] - Unable to get srt text from PMS");
        }
    } else {
        Main.debug("[Subtitle.js] - Unable to get srt text from PMS");
    }

};

Subtitle.parseLine = function (st) {
    try {
        if (st.length >= 2) {
            var i = Subtitle.strip(st[1].split(' --> ')[0]);
            var o = Subtitle.strip(st[1].split(' --> ')[1]);
            var t = st[2];

            if (isEmpty(i) || isEmpty(o) || isEmpty(t)) {
                return false;
            }

            if (st.length > 2) {
                for (var j = 3; j < st.length; j++) {
                    t += '<br>' + st[j];
                }
            }

            // delete italic and bold style
            //t = t.replace(/<(?!\s*\/?(br)\b)[^>]+>/ig, "");

            //add extra linefeeds if needed
            var lines = null;
            var lf = "";
            var cs = "";

            if (t.indexOf("<br>") > 0)
                lf = "<br>";
            else if (t.indexOf("</br>") > 0)
                lf = "</br>";
            else if (t.indexOf("<BR>") > 0)
                lf = "<BR>";
            else if (t.indexOf("</BR>") > 0)
                lf = "</BR>";

            if (lf === "") //there are no linefeeds
            {
                cs = "<br><br>" + t;
            }
            else {
                lines = t.split(lf);

                //Main.debug("[Subtitle.js] - lines:" + lines + " - " + t);

                if (lines.length === 2)
                    cs = "<br>" + t;
                else
                    cs = t;
            }
            //Main.debug("[Subtitle.js] - cs:" + cs);

            var is = Subtitle.toSeconds(i);
            var os = Subtitle.toSeconds(o);

            if (cs.length > 0) {
                var newsub = { i: is, o: os, t: cs };
                Subtitle.subtitles.push(newsub);
            }
        }
    }
    catch (err) {
        Main.debug("[Subtitle.js] - parseLine on Subtitles failed: [" + err + "] in line : " + st, 1);
    }
};

Subtitle.onSubTitle = function (time) {
    try {
        if ((Subtitle.loading === false) && (Subtitle.enabled === true) && Player.state !== Player.STOPPED) {
            var currentTime = time / 1000;
            var sub = -1;
            var s;
            var from = 0;

            if (currentTime === Subtitle.currentSubTime) {
                currentTime = currentTime + 0.5;
            }

            Subtitle.currentSubTime = time / 1000;

            //check if we did a rewind/restart
            if (Subtitle.subtitles[Subtitle.currentSubtitle]) {
                if (parseFloat(Subtitle.subtitles[Subtitle.currentSubtitle].i) > parseFloat((Subtitle.subSyncOffset + currentTime))) {
                    from = 0;
                } else {
                    from = Subtitle.currentSubtitle;
                }
            }

            for (s = from; s < Subtitle.subtitles.length; s++) {
                if (Subtitle.subtitles[s]) {
                    if (parseFloat(Subtitle.subtitles[s].i) > parseFloat((Subtitle.subSyncOffset + currentTime))) {
                        //Main.debug("[Subtitle.js] - from: " + from + " s: " + Subtitle.subtitles[s].i + " time: " + currentTime + " text: " +Subtitle.subtitles[s].t);
                        break;
                    }
                    sub = s;
                }
            }

            if (sub !== -1) {
                if (sub !== Subtitle.currentSubtitle && !isEmpty(Subtitle.subtitles[sub].t)) {
                    //Main.debug("[Subtitle.js] - s: " + parseFloat(Subtitle.subtitles[sub].i) + " time: " + parseFloat(Subtitle.subSyncOffset + currentTime));
                    widgetAPI.putInnerHTML(document.getElementById("srtSubtitle"), Subtitle.subtitles[sub].t);
                    //setting the innerHTML resets some stuff, so apply the color again , also for <i> and <b> texts
                    $("#srtSubtitle, #srtSubtitle i, #srtSubtitle b").css("color", Player.subColor);
                    Subtitle.currentSubtitle = sub;

                    //Main.debug("Sub: " + Subtitle.subtitles[sub].i + " time: " + currentTime);
                }
                else if (parseFloat(Subtitle.subtitles[sub].o) < parseFloat((Subtitle.subSyncOffset + currentTime))) {
                    widgetAPI.putInnerHTML(document.getElementById("srtSubtitle"), "");
                }
            }
        }
        else {
            widgetAPI.putInnerHTML(document.getElementById("srtSubtitle"), "");
        }
    }
    catch (err) {
        Main.debug("[Subtitle.js] - Error displaying subtitle: [" + err + "]", 1);
    }
};

/*
 ###############################
 #
 #       PRIVATE FUNCTIONS
 #
 ###############################
 */

Subtitle.strip = function (s) {
    return s.replace(/^\s+|\s+$/g, "");
};

Subtitle.toSeconds = function (t) {
    var s = 0.0;
    if (t) {
        var p = t.split(':');
        for (var i = 0; i < p.length; i++)
            s = s * 60 + parseFloat(p[i].replace(',', '.'));
    }
    return s;
};