/*
 ###############################
 #
 #       Strings
 #
 ###############################
 */

String.prototype.startsWith = function (str) {
    return (this.indexOf(str) === 0);
};

String.prototype.endsWith = function (str) {
    return this.indexOf(str, this.length - str.length) !== -1;
};

String.prototype.contains = function (str) {
    if (this.length > 1) {
        return (this.indexOf(str) >= 0);
    } else {
        return false;
    }
};

String.prototype.trim = function () {
    return this.replace(/^\s*/, "").replace(/\s*$/, "");
};

String.prototype.capitalize = function () {
    if (this.length > 1) {
        return this.charAt(0).toUpperCase() + this.slice(1).toLowerCase();
    } else {
        return this;
    }
};

String.prototype.replaceAscii = function () {
    return this.replace(/&#(\d+);/g, function (m, n) {
        return String.fromCharCode(n);
    });
};

String.prototype.replaceHtml = function () {
    var str;
    try {
        str = decodeURI(this);
    } catch (e) {
        str = this;
    }
    str = str.replace(/\r\n/g, "");
    str = str.replace(/\n/g, "");
    str = str.replace(/&#10;/g, "");
    str = str.replace(/&#13;/g, "");
    str = str.replace(/&amp;/g, "&");
    str = str.replace(/&lt;/g, "<");
    str = str.replace(/&gt;/g, ">");
    str = str.replace(/&apos;/g, "'");
    str = str.replace(/&quot;/g, "'");
    str = str.replace(/&#xA;/g, "");
    str = str.replace(/&#xD;/g, "");
    str = str.replace(/&#10;/g, "");
    str = str.replace(/&#13;/g, "");
    return str;
};

String.prototype.leftFrom = function (str, offset) {
    var os = (offset) ? offset : 0;
    var end = this.indexOf(str);
    if (end === -1)
        return this;
    else
        return this.substring(0, end + os);
};

String.prototype.leftFromLast = function (str, offset) {
    var os = (offset) ? offset : 0;
    var end = this.lastIndexOf(str);
    if (end === -1)
        return this;
    else
        return this.substring(0, end + os);
};


String.prototype.rightFromLast = function (str, offset) {
    var os = (offset) ? offset : 0;
    var start = this.lastIndexOf(str) + str.length;
    if (start === -1)
        return this;
    else
        return this.substring(start + os);
};

String.prototype.between = function (a, b, aoffset, boffset) {
    var aos = (aoffset) ? aoffset : 0;
    var bos = (boffset) ? boffset : 0;
    var start = this.indexOf(a) + a.length + aos;
    if (start >= 0) {
        if (b) {
            var end = this.indexOf(b, start) + bos;
            return this.substring(start, end);
        }
        else //get the rest of the string
        {
            return this.substring(start);
        }
    }
    else {
        return this;
    }
};

String.prototype.toTimeString = function () {
    try {
        var secNumb = parseInt(this, 10) / 1000;
        var hours = Math.floor(secNumb / 3600);
        var minutes = Math.floor((secNumb - (hours * 3600)) / 60);
        var seconds = Math.floor(secNumb - (hours * 3600) - (minutes * 60));
        var time;

        if (minutes < 10) {
            minutes = "0" + minutes;
        }
        if (seconds < 10) {
            seconds = "0" + seconds;
        }

        if (hours === "0" && minutes === "00") {
            time = seconds + " sec";
        } else if (hours === "0") {
            time = minutes + ':' + seconds;
        } else {
            time = hours + ':' + minutes + ':' + seconds;
        }

        return time;
    } catch (e) {
        return "";
    }
};

String.prototype.replaceQueryVariable = function (keyString, replaceString) {
    var query = this;
    if (query.indexOf("?") > -1) {
        var parts = query.split("?");
        var vars = parts[1].split("&");
        for (var i = 0; i < vars.length; i++) {
            var pair = vars[i].split("=");
            if (pair[0] === keyString) {
                vars[i] = pair[0] + "=" + replaceString;
            }
        }
        return parts[0] + "?" + vars.join("&");
    } else {
        return query;
    }
};

String.prototype.join = function (path) {
    if (this.endsWith('/')) {
        if (path.startsWith('/')) {
            return this + path.substr(1);
        } else {
            return this + path;
        }
    } else {
        if (path.startsWith('/')) {
            return this + path;
        } else {
            return this + "/" + path;
        }
    }
};

String.prototype.isValidIPV4 = function () {
    var value = this;
    var split = value.split('.');
    if (split.length !== 4)
        return false;

    for (var i = 0; i < split.length; i++) {
        var s = split[i];
        if (s.length === 0 || isNaN(s) || s < 0 || s > 255)
            return false;
    }
    return true;
};

/*
 ###############################
 #
 #       Numbers
 #
 ###############################
 */

Number.prototype.mod = function (base) {
    var result = this % base;
    if (result < 0 && base > 0) {
        result += base;
    }
    return result;
};

Number.prototype.toTimeString = function () {
    try {
        var secNumb = parseInt(this, 10) / 1000;
        var hours = Math.floor(secNumb / 3600);
        var minutes = Math.floor((secNumb - (hours * 3600)) / 60);
        var seconds = Math.floor(secNumb - (hours * 3600) - (minutes * 60));
        var time;

        //if (hours < 10) { hours = "0" + hours; }
        if (minutes < 10) {
            minutes = "0" + minutes;
        }
        if (seconds < 10) {
            seconds = "0" + seconds;
        }

        if (hours === "0" && minutes === "00") {
            time = seconds + " sec";
        }
        else if (hours === "0") {
            time = minutes + ':' + seconds;
        }
        else {
            time = hours + ':' + minutes + ':' + seconds;
        }

        return time;
    } catch (e) {
        return "";
    }
};

function isNumber(n) {
    return !isNaN(parseFloat(n)) && isFinite(n);
}
/*
 ###############################
 #
 #       Time
 #
 ###############################
 */

function pause(ms) {
    ms += new Date().getTime();
    while (new Date() < ms) {
    }
}

//Override for setTimeout to allow adding more parameters that keep their values from the point they are added
//else values will be evaluated at the time the function is executed
var _st = window.setTimeout;
window.setTimeout = function (fRef, mDelay) {
    try {
        if (typeof fRef === "function") {
            var argu = Array.prototype.slice.call(arguments, 2);
            var f = (function () {
                fRef.apply(null, argu);
            });
            return _st(f, mDelay);
        }
        return _st(fRef, mDelay);
    } catch (e) {
        return _st(fRef, mDelay);
    }
};

/*
 ###############################
 #
 #      Scrolling text
 #
 ###############################
 */

//Scrolling text, instead of jquery.animate in case jQuery.fx is disabled
$.fn.outerScrollHeight = function (includeMargin) {
    var element = this[0];
    var jElement = $(element);
    var totalHeight = element.scrollHeight; //includes padding
    totalHeight += jElement.outerHeight(includeMargin) - jElement.innerHeight();
    return totalHeight;
};

var scrollText = {
    dlyTimer: 0,
    intervalTimer: 0,
    init: function (o, dividerHeight) {
        var mde, id, srt, ud, p, obj, sz, clone, nu, z0;
        //noinspection CommaExpressionJS
        mde = o.Mode,
            mde = typeof (mde) === 'string' && mde.charAt(0).toUpperCase() === 'H' ? ['left', 'offsetWidth', 'top', 'width'] : ['top', 'offsetHeight', 'left', 'height'],
            id = o.ID,
            srt = o.StartDelay,
            ud = o.StartDirection,
            p = document.getElementById(id),
            obj = p.getElementsByTagName('DIV')[0],
            sz = dividerHeight, //obj[mde[1]],
            nu = Math.ceil(p[mde[1]] / sz) + 1,
            z0 = 1;
        p.style.overflow = 'hidden';
        obj.style.position = 'absolute';
        obj.style[mde[0]] = '0px';
        obj.style[mde[3]] = sz + 'px';
        for (; z0 < nu; z0++) {
            clone = obj.cloneNode(true);
            clone.style[mde[0]] = sz * z0 + 'px';
            clone.style[mde[2]] = '0px';
            obj.appendChild(clone);
        }
        o = this['zxc' + id] = {
            obj: obj,
            mde: mde[0],
            sz: sz * (z0 - 1)
        };
        clearTimeout(o.dly);
        if (typeof (srt) === 'number') {
            o.dly = setTimeout(function () {
                scrollText.scroll(id, typeof (ud) === 'number' ? ud : -1);
            }, srt);
            clearTimeout(this.dlyTimer);
            this.dlyTimer = o.dly;
        } else {
            this.scroll(id, 0);
        }
    },

    scroll: function (id, ud) {
        var oop = this,
            o = this['zxc' + id],
            p;
        if (o) {
            ud = typeof (ud) === 'number' ? ud : 0;
            clearTimeout(o.dly);
            p = parseInt(o.obj.style[o.mde], 10) + ud;
            if ((ud > 0 && p > 0) || (ud < 0 && p < -o.sz)) {
                p += o.sz * (ud > 0 ? -1 : 1);
            }
            o.obj.style[o.mde] = p + 'px';
            o.dly = setTimeout(function () {
                oop.scroll(id, ud);
            }, 200);
            clearTimeout(this.intervalTimer);
            this.intervalTimer = o.dly;
        }
    },

    stop: function () {
        clearTimeout(this.intervalTimer);
        clearTimeout(this.dlyTimer);
    }

};

/*
 ###############################
 #
 #      Image Fade
 #
 ###############################
 */

function fadeOut(obj, duration, callback) {
    window.clearInterval(obj.fadeOutTimer);
    var opacity = 100;
    obj.fadeOutTimer = window.setInterval(function () {
        opacity -= 5;
        if (opacity) {
            //Main.debug("[Utils.js] - fadeOut: " + opacity);
            obj.style.filter = 'alpha(opacity=' + opacity + ')';
            obj.style.opacity = opacity / 100;
        } else {
            window.clearInterval(obj.fadeOutTimer);
            obj.style.display = 'none';
            if (callback) {
                callback();
            }
        }
    }, duration);
}

function fadeIn(obj, duration, callback) {
    window.clearInterval(obj.fadeInTimer);
    obj.style.display = 'block';
    obj.style.opacity = 0;
    var opacity = 0;
    obj.fadeInTimer = window.setInterval(function () {
        opacity += 5;
        if (opacity <= 100) {
            //Main.debug("[Utils.js] - fadeIn: " + opacity);
            obj.style.filter = 'alpha(opacity=' + opacity + ')';
            obj.style.opacity = opacity / 100;
        } else {
            window.clearInterval(obj.fadeInTimer);
            obj.style.display = 'block';
            if (callback) {
                callback();
            }
        }
    }, duration);
}

/*
 ###############################
 #
 #      $ extentions
 #
 ###############################
 */

$.fn.putInnerHTML = function (s) {
    widgetAPI.putInnerHTML(this[0], s);
    return this;
};

/*
 ###############################
 #
 #      Helper functions
 #
 ###############################
 */

function isEmpty(obj) {
    if (typeof obj === 'undefined' || obj === null || obj === '') return true;
    if (typeof obj === 'number' && isNaN(obj)) return true;
    if (obj instanceof Date && isNaN(Number(obj))) return true;
    return false;
}

function isPrivate(ip) {
    var chk = new RegExp("(^127\.0\.0\.1)|(^192\.168\.)|(^10\.)|(^172\.1[6-9]\.)|(^172\.2[0-9]\.)|(^172\.3[0-1]\.)");
    return (ip.match(chk) === null) ? false : true;
}

function getQueryString(name) {
    var query = window.location.search;
    if (query.indexOf("?") > -1) {
        var parts = query.split("?");
        var vars = parts[1].split("&");
        for (var i = 0; i < vars.length; i++) {
            var pair = vars[i].split("=");
            if (pair[0] === name) {
                return pair[1];
            }
        }
    } else {
        return null;
    }
}
